@@ -115,15 +115,24 @@ bool DynamicRayManagementPass::runOnFunction(Function& F)
 
     bool changed = false;
 
+    bool hasDiscard = llvm::any_of(
+        instructions(F),
+        [](auto& I) {
+            return isDiscardInstruction(&I);
+        }
+    );
+
     // Dot not process further if:
     // 1. RayTracing is not supported on this platform.
     // 2. Shader does not use RayQuery at all.
     // 3. There are more than 1 exit block.
     // 4. RayQuery needs splitting due to forced SIMD32
+    // 5. Shader has discards
     if ((m_CGCtx->platform.supportRayTracing() == false) ||
         (!m_CGCtx->hasSyncRTCalls()) ||
         (getNumberOfExitBlocks(F) > 1) ||
-        m_CGCtx->syncRTCallsNeedSplitting())
+        m_CGCtx->syncRTCallsNeedSplitting() ||
+        hasDiscard)
     {
         return false;
     }
@@ -238,7 +247,8 @@ bool DynamicRayManagementPass::requiresSplittingCheckReleaseRegion(Instruction&
     return
         isa<ContinuationHLIntrinsic>(I) ||
         isBarrierIntrinsic(&I) ||
-        isUserFunctionCall(&I);
+        isUserFunctionCall(&I) ||
+        isDiscardInstruction(&I);
 }
 
 void DynamicRayManagementPass::FindProceedsInOperands(Instruction* I, SetVector<TraceRaySyncProceedHLIntrinsic*>& proceeds, SmallPtrSetImpl<Instruction*>& cache)
@@ -993,6 +1003,18 @@ void DynamicRayManagementPass::HandleComplexControlFlow(Function& F)
                     break;
                 }
 
+                if (isDiscardInstruction(&I) && !m_CGCtx->platform.allowDivergentControlFlowRayQueryCheckRelease())
+                {
+                    rayQueryReleaseIntrinsic->eraseFromParent();
+                    rayQueryCheckIntrinsic->eraseFromParent();
+
+                    // Remove the pair from the vector in case more Barriers or External
+                    // calls are between them.
+                    m_RayQueryCheckReleasePairs.erase(m_RayQueryCheckReleasePairs.begin() + rayQueryCheckReleasePairIndex);
+
+                    break;
+                }
+
                 // The barrier/call instruction is within the RayQueryCheck-Release pair.
                 // Insert RayQueryRelease before it, and RaytQueryCheck after, to re-enable
                 // Dynamic Ray Management.
