@@ -35,8 +35,6 @@ SPDX-License-Identifier: MIT
 #include <optional>
 
 
-
-
 using namespace llvm;
 using namespace RTStackFormat;
 using namespace IGC;
@@ -91,6 +89,15 @@ void RTBuilder::setInvariantLoad(LoadInst* LI)
 
 Value* RTBuilder::getRtMemBasePtr(void)
 {
+#define STYLE(X) {                                                      \
+    using T = std::conditional_t<                                       \
+        std::is_same_v<RTStackFormat::X, RTStackFormat::Xe>,            \
+        RayDispatchGlobalData::RT::Xe, RayDispatchGlobalData::RT::Xe3>; \
+    static_assert(                                                      \
+        offsetof(RayDispatchGlobalData::RT::Xe, rtMemBasePtr) ==        \
+        offsetof(T, rtMemBasePtr)); }
+#include "RayTracingMemoryStyle.h"
+#undef STYLE
     return _get_rtMemBasePtr_Xe(VALUE_NAME("rtMemBasePtr"));
 }
 
@@ -114,6 +121,8 @@ Value* RTBuilder::getMaxBVHLevels(void)
     {
     case RTMemoryStyle::Xe:
         return _get_maxBVHLevels_Xe(VALUE_NAME("maxBVHLevels"));
+    case RTMemoryStyle::Xe3:
+        return _get_maxBVHLevels_Xe3(VALUE_NAME("maxBVHLevels"));
     }
     IGC_ASSERT(0);
     return {};
@@ -1170,6 +1179,10 @@ std::pair<uint32_t, uint32_t> RTBuilder::getSliceIDBitsInSR0() const {
     {
         return {11, 15};
     }
+    else if (Ctx.platform.GetPlatformFamily() == IGFX_XE3_CORE)
+    {
+        return {14, 17};
+    }
     else
     {
         return {12, 14};
@@ -1186,6 +1199,10 @@ std::pair<uint32_t, uint32_t> RTBuilder::getSubsliceIDBitsInSR0() const {
     {
         return {8, 9};
     }
+    else if (Ctx.platform.GetPlatformFamily() == IGFX_XE3_CORE)
+    {
+        return {8, 11};
+    }
     else
     {
         return {8, 8};
@@ -1217,7 +1234,7 @@ std::pair<uint32_t, uint32_t> RTBuilder::getDualSubsliceIDBitsInSR0() const {
 // globalDSSID is the combined value of sliceID and dssID on slice.
 Value* RTBuilder::getGlobalDSSID()
 {
-    if (isChildOfXe2)
+    if (Ctx.platform.isCoreChildOf(IGFX_XE2_HPG_CORE))
     {
         if (Ctx.platform.supportsWMTPForShaderType(Ctx.type))
         {
@@ -1236,6 +1253,13 @@ Value* RTBuilder::getGlobalDSSID()
             {
                 return emitStateRegID(dssIDBits.first, sliceIDBits.second);
             }
+            else if (Ctx.platform.isCoreChildOf(IGFX_XE3_CORE))
+            {
+                Value* sliceID = emitStateRegID(sliceIDBits.first, sliceIDBits.second);
+                Value* dssID = emitStateRegID(dssIDBits.first, dssIDBits.second);
+                Value* globalDSSID = CreateMul(sliceID, getInt32(NumDSSPerSlice));
+                return CreateAdd(globalDSSID, dssID);
+            }
             else
             {
                 Value* dssID = emitStateRegID(dssIDBits.first, dssIDBits.second);
@@ -1440,6 +1464,21 @@ Value* RTBuilder::getLeafType(
 
 
 
+Value* RTBuilder::getLeafNodeSubType(StackPointerVal* StackPointer, bool CommittedHit)
+{
+    switch (getMemoryStyle())
+    {
+    case RTMemoryStyle::Xe:
+        return this->getInt32(0);
+
+    case RTMemoryStyle::Xe3:
+        return _getLeafNodeSubType_Xe3(StackPointer, getInt1(CommittedHit),
+            VALUE_NAME("MemHit.LeafNodeSubType"));
+
+    }
+    IGC_ASSERT(0);
+    return nullptr;
+}
 
 
 CallInst* RTBuilder::CreateLSCFence(
@@ -1480,6 +1519,9 @@ Value* RTBuilder::canonizePointer(Value* Ptr)
     case RTMemoryStyle::Xe:
         VA_MSB = 48 - 1;
         break;
+    case RTMemoryStyle::Xe3:
+        VA_MSB = 57 - 1;
+        break;
     }
     constexpr uint32_t MSB = 63;
     uint32_t ShiftAmt = MSB - VA_MSB;
@@ -1554,6 +1596,10 @@ Value* RTBuilder::getSyncStackID()
     {
         return _getSyncStackID_Xe2(VALUE_NAME("SyncStackID"));
     }
+    else if (PlatformInfo.eRenderCoreFamily == IGFX_XE3_CORE)
+    {
+        return _getSyncStackID_Xe3(VALUE_NAME("SyncStackID"));
+    }
     else
     {
         IGC_ASSERT_MESSAGE(0, "Invalid Product Family for SyncStackID");
@@ -1597,10 +1643,12 @@ static const char* RaytracingTypesMDName = "igc.magic.raytracing.types";
 // and add create a getter function below to retrieve it.
 enum class RaytracingType
 {
-    RTStack2Stateless     = 0,
-    RTStack2Stateful      = 1,
-    RayDispatchGlobalData = 2,
-    SWHotZone             = 3,
+    AsyncRTStack2Stateless     = 0,
+    AsyncRTStack2Stateful      = 1,
+    RayDispatchGlobalData      = 2,
+    SWHotZone                  = 3,
+    SyncRTStack2Stateless      = 4,
+    SyncRTStack2Stateful       = 5,
     NUM_TYPES
 };
 
@@ -1710,8 +1758,18 @@ Type* RTBuilder::getRTStack2PtrTy(
             AddrSpace);
     };
 
-    auto RTType = (Mode == RTBuilder::STATELESS) ?
-        RaytracingType::RTStack2Stateless : RaytracingType::RTStack2Stateful;
+    auto RTType = RaytracingType::NUM_TYPES;
+
+    if (async)
+    {
+        RTType = (Mode == RTBuilder::STATELESS) ?
+            RaytracingType::AsyncRTStack2Stateless : RaytracingType::AsyncRTStack2Stateful;
+    }
+    else
+    {
+        RTType = (Mode == RTBuilder::STATELESS) ?
+            RaytracingType::SyncRTStack2Stateless : RaytracingType::SyncRTStack2Stateful;
+    }
 
     return lazyGetRTType(*Ctx.getModule(), RTType, addTy);
 }
@@ -1831,6 +1889,10 @@ ConstantInt* RTBuilder::supportStochasticLod()
     return getInt1(Ctx.platform.supportStochasticLod());
 }
 
+ConstantInt* RTBuilder::isRayQueryReturnOptimizationEnabled()
+{
+    return getInt1(Ctx.platform.isRayQueryReturnOptimizationEnabled());
+}
 
 
 GenIntrinsicInst* RTBuilder::createDummyInstID(Value* pSrcVal)
@@ -1892,6 +1954,16 @@ void RTBuilder::createTraceRayInlinePrologue(
             ComparisonValue,
             TMax);
         break;
+    case RTMemoryStyle::Xe3:
+        _createTraceRayInlinePrologue_Xe3(
+            StackPtr,
+            RayInfo,
+            RootNodePtr,
+            RayFlags,
+            InstanceInclusionMask,
+            ComparisonValue,
+            TMax);
+        break;
     }
 }
 
@@ -1908,6 +1980,11 @@ void RTBuilder::emitSingleRQMemRayWrite(
             SMStackPtr,
             VAdapt{ *this, singleRQProceed });
         break;
+    case RTMemoryStyle::Xe3:
+        _emitSingleRQMemRayWrite_Xe3(
+            HWStackPtr,
+            SMStackPtr);
+        break;
     }
 }
 
@@ -1924,6 +2001,12 @@ void RTBuilder::copyMemHitInProceed(
             SMStackPtr,
             VAdapt{ *this, singleRQProceed });
         break;
+    case RTMemoryStyle::Xe3:
+        _copyMemHitInProceed_Xe3(
+            HWStackPtr,
+            SMStackPtr,
+            VAdapt{ *this, singleRQProceed });
+        break;
     }
 }
 
