@@ -58,8 +58,6 @@ private:
     static constexpr char *PrintfFuncName = "printf";
 
 
-    bool isChildOfXe2 = false;
-
     // Field for explicit GlobalBufferPtr - used on OpenCL path.
     Value* GlobalBufferPtr = nullptr;
     bool DisableRTGlobalsKnownValues = false;
@@ -75,9 +73,8 @@ private:
                 enabledSlices++;
             }
         }
-        isChildOfXe2 = Ctx.platform.isCoreChildOf(IGFX_XE2_HPG_CORE);
 
-        if (isChildOfXe2 || Ctx.platform.isProductChildOf(IGFX_PVC))
+        if (Ctx.platform.isCoreChildOf(IGFX_XE2_HPG_CORE) || Ctx.platform.isProductChildOf(IGFX_PVC))
         {
             NumDSSPerSlice = SysInfo.MaxSubSlicesSupported / std::max(SysInfo.MaxSlicesSupported, enabledSlices);
             EuCountPerDSS = SysInfo.MaxEuPerSubSlice;
@@ -254,6 +251,8 @@ public:
     Value* getStatelessScratchPtr(void);
     Value* getLeafType(StackPointerVal* StackPointer, bool CommittedHit);
     Value* getIsFrontFace(StackPointerVal* StackPointer, IGC::CallableShaderTypeMD ShaderTy);
+    // Xe3: memhit->leafNodeSubType
+    Value* getLeafNodeSubType(StackPointerVal* StackPointer, bool CommittedHit);
 
     Value* CreateSyncStackPtrIntrinsic(Value* Addr, Type* PtrTy, bool AddDecoration);
 
@@ -459,6 +458,7 @@ public:
 
     ConstantInt* supportStochasticLod();
 
+    ConstantInt* isRayQueryReturnOptimizationEnabled();
 
 
     GenIntrinsicInst* createDummyInstID(Value* pSrcVal);
