@@ -576,6 +576,391 @@ struct InstanceLeaf<Xe>
     } part1;
 };
 
+template <>
+struct MemHit<Xe3>
+{
+    enum class Bits : uint8_t
+    {
+        u = 24,
+        v = 24,
+        hitGroupIndex0 = 8,
+        hitGroupIndex1 = 8,
+        hitGroupIndex2 = 6,
+        hitGroupIndex3 = 6,
+        primIndexDelta = 5,
+        pad1 = 7,
+        leafNodeSubType = 4,
+        valid = 1,
+        leafType = 3,
+        primLeafIndex = 4,
+        bvhLevel = 3,
+        frontFace = 1,
+        done = 1,
+        needSWSTOC = 1,
+        reserved = 2,
+
+        primLeafPtr = 58,
+        instLeafPtr = 58,
+    };
+
+    enum class Offset : uint8_t
+    {
+        done = 28,
+    };
+
+    using T = uint32_t;
+
+    float    t;                                        // hit distance of current hit (or initial traversal distance)
+
+    uint32_t u              : (T)Bits::u;              // barycentric u hit coordinate stored as 24 bit unorm
+    uint32_t hitGroupIndex0 : (T)Bits::hitGroupIndex0; // 1st bits of hitGroupIndex
+
+    uint32_t v              : (T)Bits::v;              // barycentric v hit coordinate stored as 24 bit unorm
+    uint32_t hitGroupIndex1 : (T)Bits::hitGroupIndex1; // 2nd bits of hitGroupIndex
+
+    uint32_t primIndexDelta  : (T)Bits::primIndexDelta;  // prim index delta for compressed meshlets and quads
+    uint32_t pad1            : (T)Bits::pad1;            // MBZ
+    uint32_t leafNodeSubType : (T)Bits::leafNodeSubType; // sub-type of leaf node
+    uint32_t valid           : (T)Bits::valid;           // set if there is a hit
+    uint32_t leafType        : (T)Bits::leafType;        // type of node primLeafPtr is pointing to
+    uint32_t primLeafIndex   : (T)Bits::primLeafIndex;   // index of the hit primitive inside the leaf
+    uint32_t bvhLevel        : (T)Bits::bvhLevel;        // the instancing level at which the hit occured
+    uint32_t frontFace       : (T)Bits::frontFace;       // whether we hit the front-facing side of a triangle (also used to pass opaque flag when calling intersection shaders)
+    uint32_t done            : (T)Bits::done;            // used in sync mode to indicate that traversal is done
+    uint32_t needSWSTOC      : (T)Bits::needSWSTOC;      // If set, any-hit shader must perform a SW fallback STOC test
+    uint32_t reserved        : (T)Bits::reserved;        // unused bit
+
+    uint64_t hitGroupIndex2 : (T)Bits::hitGroupIndex2; // 3rd bits of hitGroupIndex
+    uint64_t primLeafPtr    : (T)Bits::primLeafPtr;    // pointer to BVH leaf node (MSBs of 64b pointer aligned to 64B)
+
+    uint64_t hitGroupIndex3 : (T)Bits::hitGroupIndex3; // 4th bits of hit group index
+    uint64_t instLeafPtr    : (T)Bits::instLeafPtr;    // pointer to BVH instance leaf node (MSBs of 64b pointer aligned to 64B)
+};
+static_assert(sizeof(MemHit<Xe3>) == 32, "MemHit has to be 32 bytes large");
+
+template <>
+struct MemRay<Xe3>
+{
+    // 32 B
+    Vec3f org;         // the origin of the ray
+    Vec3f dir;         // the direction of the ray
+    float tnear;       // the start of the ray
+    float tfar;        // the end of the ray
+
+    using T = uint32_t;
+    enum class Bits : uint8_t
+    {
+        rootNodePtr           = 64,
+        instLeafPtr           = 64,
+        rayFlags              = 16,
+        rayMask               = 8,
+        ComparisonValue       = 7,
+        pad1                  = 1,
+        missShaderIndex       = 16,
+        shaderIndexMultiplier = 4,
+        pad2                  = 4,
+        internalRayFlags      = 8,
+    };
+
+    uint64_t rootNodePtr : (T)Bits::rootNodePtr;  // root node to start traversal at (64-byte alignment)
+    union
+    {
+        // This just an alias for 16-bit type.
+        // As we must avoid structure members alignment, to properly generate
+        // offsets in MemRayStructure, rayFlags has 64bit type. To simplify the
+        // access, a 16bit alias is created (Ray0.flagsFromTraceRay16BitTypeAlias.rayFlags).
+        // It points to same part of MemHit structure as just Ray0.flagsFromTraceRay.rayFlags.
+        struct
+        {
+            uint16_t rayFlags;
+            uint16_t uw1;
+            uint16_t uw2;
+            uint16_t uw3;
+        } flagsFromTraceRay16BitTypeAlias;
+
+        struct
+        {
+           // This is just an alias for instLeafPtr used to access
+           // RayFlags written by the application in the shader, before the TraceRay.
+           // This structure is used to access Ray0.flagsFromTraceRay.rayFlags
+           // from the below RayFlags description.
+            uint64_t rayFlags;
+        } flagsFromTraceRay;
+
+        uint64_t instLeafPtr : (T)Bits::instLeafPtr;  // the pointer to instance leaf in case we traverse an instance (64-bytes alignment)
+    };
+
+    // RayFlags:
+    //
+    // There are 3 sets of flags, from the application point of view,
+    // available during the ray traversal:
+    // 1. RayFlags set in the shader before the TraceRay functions is called.
+    // 2. Flags applied to the entire Pipeline.
+    // 3. Flags applied per instances and geometry in the BVH
+    //
+    // All the Ray data is stored in the MemRayStructures which have two instances:
+    // 1. TopLevel (Ray0)
+    // 2. BottomLevel (Ray1)
+    //
+    // TopLevel is written by SW when the Ray is cast (before calling TraceRay).
+    // When the traversal reaches the instance and enters related BottomLevel BVH,
+    // TopLevel MemRay is copied by HW. Flags from entered instance are applied to
+    // the RayFlags.
+    // The BottomLevel MemRay is written only by HW, when AnyHit or Intersection
+    // Shaders are called.
+    //
+    // As HW can only apply flags from TracRay (1) and the BVH (3), flags from
+    // the Pipeline (2) has to applied by SW. It is done by applying them to
+    // the flags set before the TraceRay (1). But the application can read
+    // both the Pipeline flags (2) and the RayFlags (1) independently.
+    // So the original RayFlags (1) are additionally stored in the MemRay memory.
+    // As there is no dedicated space for them, the instLeafPtr from TopLevel MemRay
+    // is used, as it is irrelevant to the traversal at the TopLevel.
+    //
+    // When the AnyHit or Intersection shader are called, there are 3 sets of flags
+    // available:
+    // 1. RayFlags combined with Pipeline Flags - Ray0.rayFlags
+    // 2. RayFlags - Ray0.instLeafPtr aka Ray0.flagsFromTraceRay.rayFlags
+    // 3. RayFlags + Pipeline Flags + Instance/GeometryFlags - Ray1.rayFlags
+    //
+    // rayFlags defined here is used to access:
+    // 1. Ray0.rayFlags
+    // 2. Ray1.rayFlags
+    //
+    // Additionally an alias for 16-bit access is added.
+    // As we must avoid structure members alignment, to properly generate
+    // offsets in MemRayStructure, rayFlags has 64bit type. To simplify the
+    // access, a 16bit alias is created (rayFlags16BitAccessAlias.rayFlags).
+    // It points to same part of MemHit structure as just rayFlags.
+    union
+    {
+        struct
+        {
+            uint16_t rayFlags;
+            uint16_t uw1;
+        } rayFlags16BitTypeAlias;
+
+        struct
+        {
+            uint32_t rayFlags : (T)Bits::rayFlags;        // ray flags (see RayFlag structure)
+            uint32_t rayMask : (T)Bits::rayMask;         // ray mask used for ray masking
+            uint32_t ComparisonValue : (T)Bits::ComparisonValue; // to be compared with Instance.ComparisonMask
+            uint32_t pad1 : (T)Bits::pad1;
+        };
+    };
+
+    uint32_t hitGroupIndex;                              // hit group shader index
+
+    uint32_t missShaderIndex       : (T)Bits::missShaderIndex;       // index of miss shader to invoke on a miss
+    uint32_t shaderIndexMultiplier : (T)Bits::shaderIndexMultiplier; // shader index multiplier
+    uint32_t pad2                  : (T)Bits::pad2;
+    uint32_t internalRayFlags      : (T)Bits::internalRayFlags;      // Xe3: internal ray flags (see InternalRayFlags enum)
+
+    float time;            // ray time in range [0,1] (force to 0 for now)
+};
+static_assert(sizeof(MemRay<Xe3>) == 64, "MemRay has to be 64 bytes large");
+
+enum class InternalRayFlags : uint16_t
+{
+    NONE                            = 0x0,
+    TRIANGLE_FRONT_COUNTERCLOCKWISE = 0x1, // Xe3: switch front and back-facing triangles
+    LEVEL_ASCEND_DISABLED           = 0x2, // Xe3: disables the automatic level ascend for this level
+    SKIP_MISS_SHADER                = 0x4, // Xe3: skip execution of miss shader
+    DISABLE_STOC                    = 0x8, // Xe3: disables sub triangle opacity culling
+};
+
+template <>
+struct PrimLeafDesc<Xe3>
+{
+    enum Type : uint32_t
+    {
+        TYPE_NONE = 0,
+
+        /* For a node type of NODE_TYPE_MESHLET, the referenced leaf may
+         * still be a QuadLeaf or a Meshlet. We need this as we produce
+         * two quads instead of one meshlet when meshlet compression does
+         * not work well. */
+
+         TYPE_QUAD    = 0,
+         TYPE_MESHLET = 1,
+
+         /* For a node type of NODE_TYPE_PROCEDURAL we support enabling
+          * and disabling the opaque/non_opaque culling. */
+
+          TYPE_OPACITY_CULLING_ENABLED  = 0,
+          TYPE_OPACITY_CULLING_DISABLED = 1
+    };
+
+    enum class Bits : uint8_t
+    {
+        shaderIndex         = 24,
+        geomMask            = 8,
+
+        geomIndex           = 24,
+        subType             = 4,
+        reserved0           = 1,
+        DisableOpacityCull  = 1,
+        OpaqueGeometry      = 1,
+        IgnoreRayMultiplier = 1,
+    };
+
+    using T = uint32_t;
+
+    uint32_t shaderIndex         : (T)Bits::shaderIndex;         // shader index used for shader record calculations
+    uint32_t geomMask            : (T)Bits::geomMask;            // geometry mask used for ray masking
+
+    uint32_t geomIndex           : (T)Bits::geomIndex;           // Xe1+: the geometry index specifies the n'th geometry of the scene
+    uint32_t subType             : (T)Bits::subType;             // Xe3: geometry sub-type
+    uint32_t reserved0           : (T)Bits::reserved0;           // Xe1+: reserved bit (MBZ)
+    uint32_t DisableOpacityCull  : (T)Bits::DisableOpacityCull;  // Xe1+: disables opacity culling
+    uint32_t OpaqueGeometry      : (T)Bits::OpaqueGeometry;      // Xe1+: determines if geometry is opaque
+    uint32_t IgnoreRayMultiplier : (T)Bits::IgnoreRayMultiplier; // Xe3: ignores ray geometry multiplier
+};
+static_assert(sizeof(PrimLeafDesc<Xe3>) == 8, "PrimLeafDesc must be 8 bytes large");
+
+template <>
+struct QuadLeaf<Xe3>
+{
+    PrimLeafDesc<Xe3> leafDesc;  // the leaf header
+
+    uint32_t primIndex0;         // primitive index of first triangle (has to be at same offset as for CompressedMeshlet!)
+
+    enum class Bits : uint8_t
+    {
+        primIndex1Delta   = 5,
+        stoc1_tri0_swstoc = 1,
+        stoc1_tri1_swstoc = 1,
+        pad               = 1,
+        stoc1_tri0_opaque = 4,
+        stoc1_tri0_transp = 4,
+        j0                = 2,
+        j1                = 2,
+        j2                = 2,
+        last              = 1,
+        lastInMeshlet     = 1,
+        stoc1_tri1_opaque = 4,
+        stoc1_tri1_transp = 4,
+    };
+
+    using T = uint32_t;
+
+    uint32_t primIndex1Delta   : (T)Bits::primIndex1Delta;   // offset of primID of second triangle
+    uint32_t stoc1_tri0_swstoc : (T)Bits::stoc1_tri0_swstoc; // indicates that software STOC emulation for triangle 0 is required in AHS
+    uint32_t stoc1_tri1_swstoc : (T)Bits::stoc1_tri1_swstoc; // indicates that software STOC emulation for triangle 1 is required in AHS
+    uint32_t pad               : (T)Bits::pad;               // reserved (MBZ)
+    uint32_t stoc1_tri0_opaque : (T)Bits::stoc1_tri0_opaque; // STOC level 1 sub-triangle opaque      bits for triangle 0
+    uint32_t stoc1_tri0_transp : (T)Bits::stoc1_tri0_transp; // STOC level 1 sub-triangle transparent bits for triangle 0
+    uint32_t j0                : (T)Bits::j0;
+    uint32_t j1                : (T)Bits::j1;
+    uint32_t j2                : (T)Bits::j2;
+    uint32_t last              : (T)Bits::last;              // last quad in BVH leaf
+    uint32_t lastInMeshlet     : (T)Bits::lastInMeshlet;     // last quad in meshlet
+    uint32_t stoc1_tri1_opaque : (T)Bits::stoc1_tri1_opaque; // STOC level 1 sub-triangle opaque      bit for triangle 1
+    uint32_t stoc1_tri1_transp : (T)Bits::stoc1_tri1_transp; // STOC level 1 sub-triangle transparent bit for triangle 1
+
+    Vec3f v0;  // first vertex of first triangle
+    Vec3f v1;  // second vertex of first triangle
+    Vec3f v2;  // third vertex of first triangle
+    Vec3f v3;  // additional vertex only used for second triangle
+};
+static_assert(sizeof(QuadLeaf<Xe3>) == 64, "QuadLeaf must be 64 bytes large");
+
+struct QuadLeaf_STOC3
+{
+    QuadLeaf<Xe3> quad;
+    uint64_t fullyOpaqueBits0;       // fully opaque sub-triangle bits for triangle 0
+    uint64_t fullyTransparentBits0;  // fully transparent sub-triangle bits for triangle 0
+    uint64_t fullyOpaqueBits1;       // fully opaque sub-triangle bits for triangle 1
+    uint64_t fullyTransparentBits1;  // fully transparent sub-triangle bits for triangle 1
+
+    uint64_t stoc3_tri0_addr;      // Xe3: address of full STOC bits for triangle 0
+    uint8_t  stoc3_tri0_level;     // Xe3: the recursion level for STOC bits for triangle 0 (4-12)
+
+    uint8_t  stoc3_tri0_swstoc : 1;// Xe3: indicates that software STOC emulation for triangle 0 is required in AHS
+    uint8_t  _reserved0        : 7;
+
+    uint8_t  reserved0[6];
+
+    uint64_t stoc3_tri1_addr;      // Xe3: address of full STOC bits for triangle 1
+    uint8_t  stoc3_tri1_level;     // Xe3: the recursion level for STOC bits for triangle 1 (4-12)
+
+    uint8_t  stoc3_tri1_swstoc : 1;// Xe3: indicates that software STOC emulation for triangle 1 is required in AHS
+    uint8_t  _reserved1        : 7;
+
+    uint8_t  reserved1[6];
+};
+static_assert(sizeof(QuadLeaf_STOC3) == 128, "QuadLeaf_STOC3 must be 128 bytes large");
+
+template <>
+struct InstanceLeaf<Xe3>
+{
+    /* first 64 bytes accessed during traversal by hardware */
+    struct Part0
+    {
+        using T = uint32_t;
+        enum class Bits : uint8_t
+        {
+            instContToHitGrpIndex = 24,
+            geomMask              = 8,
+            instFlags             = 8,
+            ComparisonMode        = 1,
+            ComparisonValue       = 7,
+            pad0                  = 8,
+            subType               = 3,
+            pad1                  = 2,
+            DisableOpacityCull    = 1,
+            OpaqueGeometry        = 1,
+            IgnoreRayMultiplier   = 1,
+        };
+
+        uint32_t instanceContributionToHitGroupIndex : (T)Bits::instContToHitGrpIndex; // Xe3: instance contribution to hit group index
+        uint32_t geomMask                            : (T)Bits::geomMask;             // Xe1+: geometry mask used for ray masking
+
+        uint32_t instFlags            : (T)Bits::instFlags;           // Xe3: flags for the instance (see InstanceFlags)
+        uint32_t ComparisonMode       : (T)Bits::ComparisonMode;      // Xe3: 0 for <=, 1 for > comparison
+        uint32_t ComparisonValue      : (T)Bits::ComparisonValue;     // Xe3: to be compared with ray.ComparionMask
+        uint32_t pad0                 : (T)Bits::pad0;                // reserved (MBZ)
+        uint32_t subType              : (T)Bits::subType;             // Xe3: geometry sub-type
+        uint32_t pad1                 : (T)Bits::pad1;                // reserved (MBZ)
+        uint32_t DisableOpacityCull   : (T)Bits::DisableOpacityCull;  // Xe1+: disables opacity culling
+        uint32_t OpaqueGeometry       : (T)Bits::OpaqueGeometry;      // Xe1+: determines if geometry is opaque
+        uint32_t IgnoreRayMultiplier  : (T)Bits::IgnoreRayMultiplier; // Xe3: ignores ray geometry multiplier
+
+        uint64_t startNodePtr;                                        // Xe3: 64 bit start node of instanced object
+
+
+        // Note that the hardware swaps the translation components of the
+        // world2obj and obj2world matrices, and uses column-major instead of row-major.
+
+        // DXR and Vulkan specify transform matrices in row-major order.
+        // A 3x4 row-major matrix from the API maps to HWInstanceLeaf layout as shown:
+        //     | vx[0] vy[0] vz[0] p[0] |
+        // M = | vx[1] vy[1] vz[1] p[1] |
+        //     | vx[2] vy[2] vz[2] p[2] |
+
+        Vec3f world2obj_vx;   // 1st col of Worl2Obj transform
+        Vec3f world2obj_vy;   // 2nd col of Worl2Obj transform
+        Vec3f world2obj_vz;   // 3rd col of Worl2Obj transform
+        Vec3f obj2world_p;    // translation of Obj2World transform (on purpose in first 64 bytes)
+    } part0;
+
+    /* second 64 bytes accessed during shading */
+    struct Part1
+    {
+        uint64_t bvhPtr : 48;   // pointer to BVH where start node belongs too
+        uint64_t pad    : 16;   // unused bits (explicit padding)
+
+        uint32_t instanceID;    // user defined value per DXR spec
+        uint32_t instanceIndex; // geometry index of the instance (n'th geometry in scene)
+
+        Vec3f obj2world_vx;     // 1st col of Obj2World transform
+        Vec3f obj2world_vy;     // 2nd col of Obj2World transform
+        Vec3f obj2world_vz;     // 3rd col of Obj2World transform
+        Vec3f world2obj_p;      // translation of World2Obj transform
+    } part1;
+};
+static_assert(sizeof(InstanceLeaf<Xe3>) == 128, "InstanceLeaf must be 128 bytes large");
+
 
 // HW stack
 template <typename GenT, uint32_t MaxBVHLevels>
@@ -730,6 +1115,9 @@ static_assert(sizeof(ProceduralLeaf<Xe>) == 64, "ProceduralLeaf must be 64 bytes
 
 static_assert(sizeof(QuadLeaf<Xe>) == sizeof(ProceduralLeaf<Xe>),
     "Leaves must be same size");
+static_assert(sizeof(QuadLeaf<Xe3>) == sizeof(ProceduralLeaf<Xe3>),
+    "Leaves must be same size");
+static_assert(sizeof(QuadLeaf<Xe>) == sizeof(QuadLeaf<Xe3>));
 
 constexpr uint32_t LeafSize = sizeof(QuadLeaf<Xe>);
 
@@ -774,6 +1162,8 @@ enum class InstanceFlags : uint8_t
     TRIANGLE_FRONT_COUNTERCLOCKWISE = 0x2,
     FORCE_OPAQUE = 0x4,
     FORCE_NON_OPAQUE = 0x8,
+    FORCE_2STATE_STOC = 0x10,
+    DISABLE_STOC = 0x20,
 };
 
 template <typename GenT>
@@ -856,6 +1246,27 @@ enum class RayFlags : uint16_t
 // "Only defined ray flags are propagated by the system, e.g. visible to the RayFlags() shader intrinsic."
 constexpr uint32_t RayFlagsMask = getRayFlagMask((uint32_t)RayFlags::SKIP_PROCEDURAL_PRIMITIVES);
 
+enum class RayFlags_Xe3 : uint16_t
+{
+    NONE = 0x00,
+    FORCE_OPAQUE = 0x01,                        // forces geometry to be opaque (no anyhit shader invokation)
+    FORCE_NON_OPAQUE = 0x02,                    // forces geometry to be non-opqaue (invoke anyhit shader)
+    ACCEPT_FIRST_HIT_AND_END_SEARCH = 0x04,     // terminates traversal on the first hit found (shadow rays)
+    SKIP_CLOSEST_HIT_SHADER = 0x08,             // skip execution of the closest hit shader
+    CULL_BACK_FACING_TRIANGLES = 0x10,          // back facing triangles to not produce a hit
+    CULL_FRONT_FACING_TRIANGLES = 0x20,         // front facing triangles do not produce a hit
+    CULL_OPAQUE = 0x40,                         // opaque geometry does not produce a hit
+    CULL_NON_OPAQUE = 0x80,                     // non-opaque geometry does not produce a hit
+
+    SKIP_TRIANGLES = 0x100,                     // Skip all triangle intersections and consider them as misses
+    SKIP_PROCEDURAL_PRIMITIVES = 0x200,         // Skip execution of intersection shaders for procedural primitives
+
+    FORCE_2STATE_STOC = 0x400,                  // Force 2 state sub triangle opacity mode.
+};
+
+// DXR spec:
+// "Only defined ray flags are propagated by the system, e.g. visible to the RayFlags() shader intrinsic."
+constexpr uint32_t RayFlagsMask_Xe3 = getRayFlagMask((uint32_t)RayFlags_Xe3::FORCE_2STATE_STOC);
 static_assert(sizeof(MemTravStack) == 32, "MemTravStack has to be 32 bytes large");
 
 
@@ -1114,6 +1525,32 @@ enum CANDIDATE_TYPE : uint32_t
     CANDIDATE_PROCEDURAL_PRIMITIVE
 };
 
+struct RayQueryReturnData
+{
+    enum class Bits : uint8_t
+    {
+        proceed_further = 1,
+        committedStatus = 2,
+        candidateType = 1,
+        reserved = 28,
+    };
+
+    // This union is defined for futures use.
+    // It will replace writing the RayQuery Return Value
+    // to the MemHit in ShadowMemory.
+    union
+    {
+        uint32_t rayQueryReturnDWord;
+
+        struct
+        {
+            uint32_t PROCEED_FURTHER : (uint32_t)Bits::proceed_further; // This bit when set indicates that traversal is not complete i.e. all candidates have not been searched yet.
+            COMMITTED_STATUS committedStatus : (uint32_t)Bits::committedStatus; // This bit field provides the Committed Hit's Status / Type.
+            CANDIDATE_TYPE candidateType : (uint32_t)Bits::candidateType;   // This bit indicates the candidate type for a potential hit.
+            uint32_t reserved : (uint32_t)Bits::reserved;        // Reserved
+        };
+    };
+};
 
 } // namespace RTStackFormat
 
