@@ -159,6 +159,7 @@ void CheckInstrTypes::print(llvm::raw_ostream& OS) const
     OS << "\nhasGenericAddressSpacePointers: " << g_InstrTypes.hasGenericAddressSpacePointers;
     OS << "\nhasDebugInfo: " << g_InstrTypes.hasDebugInfo;
     OS << "\nhasAtomics: " << g_InstrTypes.hasAtomics;
+    OS << "\nhasLocalAtomics: " << g_InstrTypes.hasLocalAtomics;
     OS << "\nhasDiscard: " << g_InstrTypes.hasDiscard;
     OS << "\nhasTypedRead: " << g_InstrTypes.hasTypedRead;
     OS << "\nhasTypedwrite: " << g_InstrTypes.hasTypedwrite;
@@ -311,9 +312,18 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
         case GenISAIntrinsic::GenISA_LSCAtomicFP64:
         case GenISAIntrinsic::GenISA_LSCAtomicFP32:
         case GenISAIntrinsic::GenISA_LSCAtomicInts:
+        {
             g_InstrTypes.hasAtomics = true;
             g_InstrTypes.numAtomics++;
+            Value* bufPtr = GetBufferOperand(CI);
+            if (bufPtr &&
+                bufPtr->getType()->isPointerTy() &&
+                ADDRESS_SPACE_LOCAL == bufPtr->getType()->getPointerAddressSpace())
+            {
+                g_InstrTypes.hasLocalAtomics = true;
+            }
             break;
+        }
         case GenISAIntrinsic::GenISA_discard:
             g_InstrTypes.hasDiscard = true;
             break;
@@ -329,11 +339,13 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
             g_InstrTypes.hasUniformAssumptions = true;
             break;
         case GenISAIntrinsic::GenISA_typedread:
+        case GenISAIntrinsic::GenISA_typedreadMS:
             g_InstrTypes.hasTypedRead = true;
             g_InstrTypes.numTypedReadWrite++;
             g_InstrTypes.num2DAccesses++;
             break;
         case GenISAIntrinsic::GenISA_typedwrite:
+        case GenISAIntrinsic::GenISA_typedwriteMS:
             g_InstrTypes.hasTypedwrite = true;
             g_InstrTypes.numTypedReadWrite++;
             g_InstrTypes.num2DAccesses++;
@@ -368,7 +380,9 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
             g_InstrTypes.num1DAccesses++;
             BufferType bufferType = DecodeBufferType(
                 CI->getArgOperand(0)->getType()->getPointerAddressSpace());
-            if (bufferType == UAV || bufferType == BINDLESS)
+            if (bufferType == UAV ||
+                bufferType == BINDLESS ||
+                bufferType == BINDLESS_READONLY)
             {
                 g_InstrTypes.hasStorageBufferLoad = true;
             }
@@ -380,7 +394,9 @@ void CheckInstrTypes::visitCallInst(CallInst& C)
             g_InstrTypes.num1DAccesses++;
             BufferType bufferType = DecodeBufferType(
                 CI->getArgOperand(0)->getType()->getPointerAddressSpace());
-            if (bufferType == UAV || bufferType == BINDLESS)
+            if (bufferType == UAV ||
+                bufferType == BINDLESS ||
+                bufferType == BINDLESS_WRITEONLY)
             {
                 g_InstrTypes.hasStorageBufferStore = true;
             }
@@ -564,6 +580,7 @@ void CheckInstrTypes::visitLoadInst(LoadInst& I)
         g_InstrTypes.num1DAccesses++;
         break;
     case IGC::BINDLESS:
+    case IGC::BINDLESS_READONLY:
     case IGC::SSH_BINDLESS:
         g_InstrTypes.hasStorageBufferLoad = true;
         break;
@@ -583,6 +600,7 @@ void CheckInstrTypes::visitLoadInst(LoadInst& I)
     case IGC::CONSTANT_BUFFER:
     case IGC::POINTER:
     case IGC::BINDLESS_CONSTANT_BUFFER:
+    case IGC::BINDLESS_WRITEONLY:
     case IGC::BINDLESS_TEXTURE:
     case IGC::SAMPLER:
     case IGC::BINDLESS_SAMPLER:
@@ -637,6 +655,7 @@ void CheckInstrTypes::visitStoreInst(StoreInst& I)
         g_InstrTypes.num1DAccesses++;
         break;
     case IGC::BINDLESS:
+    case IGC::BINDLESS_WRITEONLY:
     case IGC::SSH_BINDLESS:
         g_InstrTypes.hasStorageBufferStore = true;
         break;
@@ -656,6 +675,7 @@ void CheckInstrTypes::visitStoreInst(StoreInst& I)
     case IGC::CONSTANT_BUFFER:
     case IGC::POINTER:
     case IGC::BINDLESS_CONSTANT_BUFFER:
+    case IGC::BINDLESS_READONLY:
     case IGC::BINDLESS_TEXTURE:
     case IGC::SAMPLER:
     case IGC::BINDLESS_SAMPLER:
