@@ -101,6 +101,9 @@ public:
     void Select(const SSource sources[3], const DstModifier& modifier);
     void PredAdd(const SSource& pred, bool invert, const SSource sources[2], const DstModifier& modifier);
     void Mul(const SSource[2], const DstModifier& modifier);
+    void Div(const SSource[2], const DstModifier& modifier);
+    void Inv(const SSource[2], const DstModifier& modifier);
+    void Add(const SSource[2], const DstModifier& modifier);
     void FPTrunc(const SSource[2], const DstModifier& modifier);
     void Powi(const SSource[2], const DstModifier& modifier);
     void Mov(const SSource& source, const DstModifier& modifier);
@@ -707,6 +710,7 @@ public:
     void MovPhiSources(llvm::BasicBlock* bb);
 
     void InitConstant(llvm::BasicBlock* BB);
+    void emitLifetimeStartResourceLoopUnroll(llvm::BasicBlock* BB);
     void emitLifetimeStartAtEndOfBB(llvm::BasicBlock* BB);
     void emitDebugPlaceholder(llvm::GenIntrinsicInst* I);
     void emitDummyInst(llvm::GenIntrinsicInst* GII);
@@ -800,6 +804,7 @@ public:
 
     // generate loop header to process sample instruction with varying resource/sampler
     bool ResourceLoopHeader(
+        const CVariable* destination,
         ResourceDescriptor& resource,
         SamplerDescriptor& sampler,
         CVariable*& flag,
@@ -807,6 +812,7 @@ public:
         uint ResourceLoopMarker = 0,
         int* subInteration = nullptr);
     bool ResourceLoopHeader(
+        const CVariable* destination,
         ResourceDescriptor& resource,
         CVariable*& flag,
         uint& label,
@@ -868,7 +874,7 @@ public:
             }
 
             // label resource loop
-            ResourceLoopHeader(resource, sampler, flag, label, ResourceLoopMarker, &subInteration);
+            ResourceLoopHeader(currentDestination, resource, sampler, flag, label, ResourceLoopMarker, &subInteration);
         }
 
         // subInteration == 0 could mean no resource loop tag indicated
