@@ -74,6 +74,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/FPRoundingModeCoalescing.hpp"
 
 #include "Compiler/CISACodeGen/SLMConstProp.hpp"
+#include "Compiler/Legalizer/AddRequiredMemoryFences.h"
 #include "Compiler/Optimizer/OpenCLPasses/GenericAddressResolution/GenericAddressDynamicResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryUsageAnalysis.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryResolution.hpp"
@@ -312,6 +313,10 @@ void AddAnalysisPasses(CodeGenContext& ctx, IGCPassManager& mpm)
             }
         }
         mpm.add(createPromoteMemoryToRegisterPass());
+        if (IGC_IS_FLAG_DISABLED(DisableMergeAllocasPrivateMemory) && ctx.type == ShaderType::OPENCL_SHADER)
+        {
+            mpm.add(createMergeAllocas());
+        }
         if (ctx.type == ShaderType::OPENCL_SHADER &&
             !isOptDisabled &&
             IGC_IS_FLAG_ENABLED(EnableExplicitCopyForByVal))
@@ -511,11 +516,7 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         if (IGC_IS_FLAG_ENABLED(allowLICM) && ctx.m_retryManager.AllowLICM())
         {
             mpm.add(createSpecialCasesDisableLICM());
-#if !defined(IGC_LLVM_TRUNK_REVISION)
             mpm.add(llvm::createLICMPass(100, 500, true));
-#else
-            mpm.add(llvm::createLICMPass(100, 500));
-#endif
         }
         mpm.add(llvm::createLoopSimplifyPass());
     }
@@ -656,13 +657,6 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
             mpm.add(createPromoteMemoryToRegisterPass());
     }
 
-    // There's no particular reason for this exact place, but it should be after LowerGEPForPrivMem
-    if (IGC_IS_FLAG_ENABLED(EnableSplitIndirectEEtoSel))
-    {
-        mpm.add(createSplitIndirectEEtoSelPass());
-    }
-
-
     if (ctx.type == ShaderType::OPENCL_SHADER ||
         ctx.type == ShaderType::COMPUTE_SHADER)
     {
@@ -716,6 +710,10 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
     if (!(IGC_IS_FLAG_ENABLED(EnableUnmaskedFunctions) &&
           IGC_IS_FLAG_ENABLED(LateInlineUnmaskedFunc)))
     {
+        if (IGC_IS_FLAG_DISABLED(DisableMergeAllocasPrivateMemory) && ctx.type == ShaderType::OPENCL_SHADER)
+        {
+            mpm.add(createMergeAllocas());
+        }
         if (ctx.type == ShaderType::OPENCL_SHADER &&
             !isOptDisabled &&
             IGC_IS_FLAG_ENABLED(EnableExplicitCopyForByVal))
@@ -783,6 +781,8 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         mpm.add(createIGCInstructionCombiningPass());
     }
 
+
+
     if (ctx.hasSyncRTCalls())
     {
         mpm.add(createRaytracingStatefulPass());
@@ -903,11 +903,7 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         if (!fastCompile && !highAllocaPressure && !isPotentialHPCKernel && IGC_IS_FLAG_ENABLED(allowLICM) && ctx.m_retryManager.AllowLICM())
         {
             mpm.add(createSpecialCasesDisableLICM());
-#if !defined(IGC_LLVM_TRUNK_REVISION)
             mpm.add(llvm::createLICMPass(100, 500, true));
-#else
-            mpm.add(llvm::createLICMPass(100, 500));
-#endif
             mpm.add(llvm::createEarlyCSEPass());
         }
         mpm.add(createAggressiveDCEPass());
@@ -1114,6 +1110,14 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         mpm.add(new RuntimeValueLegalizationPass());
     }
 
+    if ((ctx.m_instrTypes.hasLocalLoadStore || ctx.m_instrTypes.hasLocalAtomics) &&
+        ctx.platform.hasLSC() &&
+        !ctx.platform.NeedsLSCFenceUGMBeforeEOT() && // VISA will add the fence
+        IGC_IS_FLAG_DISABLED(DisableAddRequiredMemoryFencesPass))
+    {
+        mpm.add(createAddRequiredMemoryFencesPass());
+    }
+
     mpm.add(createInstSimplifyLegacyPass());
     // This pass inserts bitcasts for vector loads/stores.
     // This pass could be moved further toward EmitPass.
@@ -1519,11 +1523,7 @@ void OptimizeIR(CodeGenContext* const pContext)
                     mpm.add(createSpecialCasesDisableLICM());
                     int licmTh = IGC_GET_FLAG_VALUE(LICMStatThreshold);
                     mpm.add(new InstrStatistic(pContext, LICM_STAT, InstrStatStage::BEGIN, licmTh));
-#if !defined(IGC_LLVM_TRUNK_REVISION)
                     mpm.add(llvm::createLICMPass(100, 500, true));
-#else
-                    mpm.add(llvm::createLICMPass(100, 500));
-#endif
                     mpm.add(new InstrStatistic(pContext, LICM_STAT, InstrStatStage::END, licmTh));
                 }
 
@@ -1541,7 +1541,8 @@ void OptimizeIR(CodeGenContext* const pContext)
 
                 mpm.add(createIGCInstructionCombiningPass());
 
-                if (IGC_IS_FLAG_ENABLED(EnableIndVarSimplification))
+                if (IGC_IS_FLAG_ENABLED(EnableIndVarSimplification) &&
+                    pContext->type == ShaderType::OPENCL_SHADER)
                 {
                     mpm.add(llvm::createIndVarSimplifyPass());
                 }
@@ -1590,11 +1591,7 @@ void OptimizeIR(CodeGenContext* const pContext)
                 if (allowLICM)
                 {
                     mpm.add(createSpecialCasesDisableLICM());
-#if !defined(IGC_LLVM_TRUNK_REVISION)
                     mpm.add(llvm::createLICMPass(100, 500, true));
-#else
-                    mpm.add(llvm::createLICMPass(100, 500));
-#endif
                 }
 
                 // Second unrolling with the same threshold.
@@ -1851,7 +1848,6 @@ void OptimizeIR(CodeGenContext* const pContext)
             mpm.add(createSROAPass());
         }
 
-        mpm.add(new TrivialLocalMemoryOpsElimination());
         if (pContext->type == ShaderType::COMPUTE_SHADER &&
             (IGC_IS_FLAG_ENABLED(RemoveUnusedTGMFence) ||
                 pContext->getModuleMetaData()->enableRemoveUnusedTGMFence))
