@@ -192,6 +192,8 @@ namespace IGC
 
     bool isUserFunctionCall(const llvm::Instruction* I);
 
+    bool isDiscardInstruction(const llvm::Instruction* I);
+
     bool IsMemLoadIntrinsic(llvm::GenISAIntrinsic::ID id);
 
     bool IsStatelessMemLoadIntrinsic(llvm::GenISAIntrinsic::ID id);
@@ -233,6 +235,8 @@ namespace IGC
     {
         return t == BINDLESS ||
             t == BINDLESS_CONSTANT_BUFFER ||
+            t == BINDLESS_READONLY ||
+            t == BINDLESS_WRITEONLY ||
             t == BINDLESS_TEXTURE;
     }
     inline bool IsSSHbindless(BufferType t)
@@ -259,6 +263,8 @@ namespace IGC
             t == CONSTANT_BUFFER ||
             t == BINDLESS ||
             t == BINDLESS_CONSTANT_BUFFER ||
+            t == BINDLESS_READONLY ||
+            t == BINDLESS_WRITEONLY ||
             t == SSH_BINDLESS ||
             t == SSH_BINDLESS_CONSTANT_BUFFER;
     }
@@ -494,8 +500,6 @@ namespace IGC
     unsigned int AppendConservativeRastWAHeader(IGC::SProgramOutput* program, SIMDMode simdmode);
     unsigned int AppendConservativeRastWAHeader(void*& pBinary, unsigned int& binarySize, SIMDMode simdmode);
 
-    bool DSDualPatchEnabled(class CodeGenContext* ctx);
-
 
     /// \brief Check whether inst precedes given position in one basic block
     inline bool isInstPrecede(
