@@ -87,6 +87,7 @@ namespace IGCOpts
 
 namespace IGC
 {
+    struct BifLLVMModule;
     class CodeGenContext;
 
     struct SProgramOutput
@@ -332,6 +333,7 @@ namespace IGC
         bool hasGenericAddressSpacePointers{};
         bool hasDebugInfo{};        //<! true only if module contains debug info !llvm.dbg.cu
         bool hasAtomics{};
+        bool hasLocalAtomics{};
         bool hasDiscard{};
         bool hasTypedRead{};
         bool hasTypedwrite{};
@@ -452,6 +454,7 @@ namespace IGC
         SIMD16_OFFSET = SIMD_INFO_RESERVED,
         SIMD32_OFFSET = SIMD_INFO_RESERVED*2,
         DUAL_SIMD8_OFFSET = SIMD_INFO_RESERVED * 3,
+        QUAD_SIMD8_DYNAMIC_OFFSET = SIMD_INFO_RESERVED*6,
     };
 
     struct SKernelProgram
@@ -975,6 +978,12 @@ namespace IGC
         unsigned int        m_constantPayloadNextAvailableGRFOffset = 0;
         ConstantPayloadInfo m_constantPayloadOffsets;
 
+        // Contains the data (bytecode, enabling bit) for BIF functions
+        // provided externally.
+        size_t m_numBifModules = 0;
+        BifLLVMModule* m_bifModules = nullptr;
+        // If this flag is enabled, STOC level emulation will be added to every AnyHitShader.
+        bool m_enableSubTriangleOpacityEmulation = false;
         void* gtpin_init = nullptr;
         bool m_hasLegacyDebugInfo = false;
         bool m_hasEmu64BitInsts = false;
@@ -1153,6 +1162,7 @@ namespace IGC
             return llvmCtxWrapper->m_allLayoutStructTypes;
         }
 
+        bool isSWSubTriangleOpacityCullingEmulationEnabled() const;
 
         unsigned int GetSIMDInfoOffset(SIMDMode simd, ShaderDispatchMode mode)
         {
@@ -1178,10 +1188,14 @@ namespace IGC
             case ShaderDispatchMode::DUAL_SIMD8:
                 offset = DUAL_SIMD8_OFFSET;
                 break;
+            case ShaderDispatchMode::QUAD_SIMD8_DYNAMIC:
+                offset = QUAD_SIMD8_DYNAMIC_OFFSET;
+                break;
 
             default:
                 break;
             }
+            IGC_ASSERT(offset < 64);
             return offset;
         }
 
