@@ -253,6 +253,8 @@ KernelArg::ArgType KernelArg::calcArgType(const ImplicitArg& arg) const
         return KernelArg::ArgType::IMPLICIT_R0;
     case ImplicitArg::PAYLOAD_HEADER:
         return KernelArg::ArgType::IMPLICIT_PAYLOAD_HEADER;
+    case ImplicitArg::PAYLOAD_HEADER_SHORT:
+        return KernelArg::ArgType::IMPLICIT_PAYLOAD_HEADER_SHORT;
     case ImplicitArg::PRIVATE_BASE:
         return KernelArg::ArgType::IMPLICIT_PRIVATE_BASE;
     case ImplicitArg::CONSTANT_BASE:
@@ -436,6 +438,11 @@ unsigned int KernelArg::getAllocateSize() const
     return int_cast<unsigned int>(llvm::alignTo(m_allocateSize, iOpenCL::DATA_PARAMETER_DATA_SIZE));
 }
 
+unsigned int KernelArg::getSize() const
+{
+    return m_allocateSize;
+}
+
 unsigned int KernelArg::getElemAllocateSize() const
 {
     return m_elemAllocateSize;
@@ -802,6 +809,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
 
             KernelArg::ArgType::RUNTIME_VALUE,
             KernelArg::ArgType::IMPLICIT_PAYLOAD_HEADER,
+            KernelArg::ArgType::IMPLICIT_PAYLOAD_HEADER_SHORT,
 
             KernelArg::ArgType::PTR_LOCAL,
             KernelArg::ArgType::PTR_GLOBAL,
@@ -928,6 +936,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
 
             KernelArg::ArgType::RUNTIME_VALUE,
             KernelArg::ArgType::IMPLICIT_PAYLOAD_HEADER,
+            KernelArg::ArgType::IMPLICIT_PAYLOAD_HEADER_SHORT,
             KernelArg::ArgType::PTR_LOCAL,
             KernelArg::ArgType::PTR_GLOBAL,
             KernelArg::ArgType::PTR_CONSTANT,
