@@ -44,6 +44,7 @@ namespace IGC
             R1,
 
             IMPLICIT_PAYLOAD_HEADER, // known as INPUT_HEADER in USC
+            IMPLICIT_PAYLOAD_HEADER_SHORT, // payload header reduced to 3xi32
 
             PTR_LOCAL,
             PTR_GLOBAL,
@@ -217,6 +218,7 @@ namespace IGC
         AccessQual                      getAccessQual()         const;
         unsigned int                    getNumComponents()      const;
         unsigned int                    getAllocateSize()       const;
+        unsigned int                    getSize()               const;
         unsigned int                    getElemAllocateSize()   const;
         size_t                          getAlignment()          const;
         bool                            isConstantBuf()         const;
