@@ -208,6 +208,8 @@ void StatelessToStateful::handleFunction(llvm::Function& F)
     m_ctx = static_cast<OpenCLProgramContext*>(getAnalysis<CodeGenContextWrapper>().getCodeGenContext());
     m_pKernelArgs = new KernelArgs(F, &(F.getParent()->getDataLayout()), pMdUtils, modMD, m_ctx->platform.getGRFSize());
 
+    m_changed = hoistLoad();
+
     findPromotableInstructions();
     promote();
 
@@ -218,6 +220,185 @@ void StatelessToStateful::handleFunction(llvm::Function& F)
     m_promotionMap.clear();
 }
 
+bool StatelessToStateful::canWriteToMemoryTill(Instruction *Till) {
+    BasicBlock *BB = Till->getParent();
+    for (auto &I : *BB) {
+        // Stop checking when we reach the PHINode
+        if (&I == Till)
+            break;
+
+        if (I.mayWriteToMemory())
+            return true;
+    }
+
+    return false;
+}
+
+// This function checks if it is safe to hoist the load instruction over the phi instruction.
+// It supports the following cases (BB3 contains load and phi instructions):
+//
+// 1)
+//   BB1  BB2
+//   |   /
+//    BB3
+// Here the function will check that there are no instructions that can write to memory in BB3 (from phi instruction till load instruction).
+//
+// 2)
+//  BB1
+//  |  \
+//  |   BB2
+//  |  /
+//   BB3
+// Here the function will check that there are no instructions that can write to memory in the whole basic block BB2.
+bool StatelessToStateful::isItSafeToHoistLoad(LoadInst *LI, PHINode *Phi)
+{
+    BasicBlock *LoadBB = LI->getParent();
+
+    // Check that the load instruction is not carried over an instruction that can write to memory in basic block with phi instruction.
+    if (canWriteToMemoryTill(LI))
+        return false;
+
+    // Iterate over incoming basic blocks of the phi instruction.
+    for (unsigned i = 0; i < Phi->getNumIncomingValues(); ++i) {
+        BasicBlock *IncomingBlock = Phi->getIncomingBlock(i);
+
+        // Skip incoming block if it leads to the load block only.
+        if (IncomingBlock->getTerminator()->getNumSuccessors() == 1)
+            continue;
+
+        // Get the successor of the incoming block that is not the load block.
+        BasicBlock* TheOtherSuccessor = nullptr;
+        auto IncomingBlockTerminator = IncomingBlock->getTerminator();
+        for (unsigned j = 0; j < IncomingBlockTerminator->getNumSuccessors(); ++j) {
+            BasicBlock *Successor = IncomingBlockTerminator->getSuccessor(j);
+            if (Successor != LoadBB) {
+                TheOtherSuccessor = Successor;
+                break;
+            }
+        }
+
+        // Iterate over phi incoming values again to check that TheOtherSuccessor is incoming block of the phi instruction.
+        bool isSuccessorIncoming = false;
+        for (unsigned k = 0; k < Phi->getNumIncomingValues(); ++k) {
+            BasicBlock *BBToCheck = Phi->getIncomingBlock(k);
+            if (BBToCheck != TheOtherSuccessor)
+                continue;
+
+            isSuccessorIncoming = true;
+
+            // Check that TheOtherSuccessor does not have any instruction that can write to memory.
+            if (canWriteToMemoryTill(TheOtherSuccessor->getTerminator()))
+                return false;
+        }
+
+        // Check that TheOtherSuccessor is incomimg block of the phi instruction.
+        if (!isSuccessorIncoming)
+            return false;
+    }
+
+    return true;
+}
+
+bool StatelessToStateful::hoistLoad()
+{
+    bool Changed = false;
+    std::vector<std::tuple<PHINode*, LoadInst*>> Container;
+    Container.reserve(128);
+
+    // Collect all load instructions that can be hoisted.
+    for (auto &BB : *m_F)
+    {
+        for (auto &I : BB)
+        {
+            auto *LI = dyn_cast<LoadInst>(&I);
+            if (!LI)
+                continue;
+
+            Value *Addr = LI->getPointerOperand();
+            if (!Addr)
+                continue;
+
+            // StatelessToStateful works only with global and constant address space pointers.
+            PointerType *PtrType = cast<PointerType>(Addr->getType());
+            if (!PtrType || (PtrType->getAddressSpace() != ADDRESS_SPACE_GLOBAL &&
+                PtrType->getAddressSpace() != ADDRESS_SPACE_CONSTANT))
+            {
+                continue;
+            }
+
+            // Sometimes result of phi instruction can be casted before load instruction.
+            PHINode *Phi = nullptr;
+            if (isa<PHINode>(Addr))
+            {
+                Phi = cast<PHINode>(Addr);
+            }
+            else if (auto *BCI = dyn_cast<BitCastInst>(Addr))
+            {
+                if (!BCI->hasOneUser())
+                    continue;
+
+                if (isa<PHINode>(BCI->getOperand(0)))
+                    Phi = cast<PHINode>(BCI->getOperand(0));
+            }
+
+            // Check that PHINode and load instructions are in the same basic block
+            if (!Phi || Phi->getParent() != &BB)
+                continue;
+
+            if (!Phi->hasOneUser())
+                continue;
+
+            // Check that the instruction is not carried over an instruction that can write to memory.
+            if (isItSafeToHoistLoad(LI, Phi))
+                Container.push_back(std::make_tuple(Phi, LI));
+        }
+    }
+
+    // Iterate over phi nodes and load instructions.
+    // Hoist the load instructions to the incoming BBs.
+    // Create new phi instruction with the hoisted load instructions as incoming values.
+    // Update uses of the original load instruction with the new phi instruction.
+    // Remove the load instruction and the bitcast inst instruction (if it exists) from the original BB.
+    for (auto &[Phi, LI] : Container)
+    {
+        IRBuilder<> Builder(Phi->getContext());
+        Builder.SetInsertPoint(Phi);
+
+        Type *LoadType = LI->getType();
+        Align LoadAlign = LI->getAlign();
+        PHINode *NewPhi = Builder.CreatePHI(LoadType, Phi->getNumIncomingValues(), "new_phi");
+
+        // Iterate over phi incoming basic blocks.
+        for (unsigned i = 0; i < Phi->getNumIncomingValues(); ++i) {
+            Value *IncomingValue = Phi->getIncomingValue(i);
+            BasicBlock *IncomingBlock = Phi->getIncomingBlock(i);
+            Instruction *Terminator = IncomingBlock->getTerminator();
+
+            Builder.SetInsertPoint(Terminator);
+            Type *IncomingPtrType = IncomingValue->getType();
+            PointerType *LoadPtrType = dyn_cast<PointerType>(LI->getPointerOperand()->getType());
+
+            // Cast the incoming value to the type of the load if it is needed.
+            Value *Cast = IncomingValue;
+            if (IncomingPtrType != LoadPtrType)
+                Cast = Builder.CreateBitCast(IncomingValue, LoadPtrType);
+
+            LoadInst *NewLoad = Builder.CreateAlignedLoad(LoadType, Cast, LoadAlign, "hoisted_" + LI->getName());
+            NewPhi->addIncoming(NewLoad, IncomingBlock);
+        }
+
+        // Erase all old instructions and update uses.
+        LI->replaceAllUsesWith(NewPhi);
+        Value *Addr = LI->getPointerOperand();
+        LI->eraseFromParent();
+        if (auto *BCI = dyn_cast<BitCastInst>(Addr))
+            BCI->eraseFromParent();
+        Phi->eraseFromParent();
+    }
+
+    return Changed;
+}
+
 Argument* StatelessToStateful::getBufferOffsetArg(Function* F, uint32_t ArgNumber)
 {
     uint32_t nImplicitArgs = m_pImplicitArgs->size();
@@ -382,21 +563,69 @@ bool StatelessToStateful::pointerIsFromKernelArgument(Value& ptr)
     return false;
 }
 
+static alignment_t determinePointerAlignment(
+    Value *Ptr,
+    const DataLayout &DL,
+    AssumptionCache *AC,
+    Instruction *InsertionPt)
+{
+    alignment_t BestAlign = 0;
+
+    // 1) Examine uses: look for loads/stores (which may carry explicit
+    //    alignment) or a GEP that reveals an ABI alignment from its element
+    //    type.
+    for (User *U : Ptr->users()) {
+      if (auto *LI = dyn_cast<LoadInst>(U)) {
+        // Load has an explicit alignment.
+        alignment_t LdAlign = LI->getAlign().value();
+        if (LdAlign > BestAlign)
+          BestAlign = LdAlign;
+      } else if (auto *SI = dyn_cast<StoreInst>(U)) {
+        // Store sets alignment only if the pointer we store into is Ptr.
+        if (SI->getPointerOperand() == Ptr) {
+          alignment_t StAlign = SI->getAlign().value();
+          if (StAlign > BestAlign)
+            BestAlign = StAlign;
+        }
+      } else if (auto *GEP = dyn_cast<GetElementPtrInst>(U)) {
+        // If the GEP's source element type is sized, we can guess an ABI
+        // alignment.
+        Type *BaseTy = GEP->getSourceElementType();
+        if (BaseTy && BaseTy->isSized()) {
+          alignment_t GEPAlign = DL.getABITypeAlign(BaseTy).value();
+          if (GEPAlign > BestAlign)
+            BestAlign = GEPAlign;
+        }
+      }
+    }
+
+    // 2) If this pointer is actually a function parameter, see if it has an
+    //    alignment attribute.
+    if (auto *Arg = dyn_cast<Argument>(Ptr))
+    {
+        if (Arg->hasAttribute(llvm::Attribute::Alignment))
+        {
+            if (MaybeAlign ArgAlign = Arg->getParamAlign())
+            {
+                alignment_t ArgAlignOrOne = ArgAlign.valueOrOne().value();
+                if (ArgAlignOrOne > BestAlign)
+                    BestAlign = ArgAlignOrOne;
+            }
+        }
+    }
+
+    // 3) Fallback: use LLVM's built-in assumption-based alignment analysis
+    //    (based on a.o. llvm.assume intrinsics).
+    Align Known = getKnownAlignment(Ptr, DL, InsertionPt, AC);
+    if (Known > BestAlign)
+        BestAlign = Known.value();
+
+    return BestAlign;
+}
+
 bool StatelessToStateful::pointerIsPositiveOffsetFromKernelArgument(
     Function* F, Value* V, Value*& offset, unsigned int& argNumber, bool ignoreSyncBuffer)
 {
-    auto getPointeeAlign = [](const DataLayout* DL, Value* ptrVal)-> alignment_t {
-        if (PointerType* PTy = dyn_cast<PointerType>(ptrVal->getType()))
-        {
-            Type* pointeeTy = IGCLLVM::getNonOpaquePtrEltTy(PTy);
-            if (!pointeeTy->isSized()) {
-                return 0;
-            }
-            return DL->getABITypeAlign(pointeeTy).value();
-        }
-        return 0;
-    };
-
     const DataLayout* DL = &F->getParent()->getDataLayout();
 
     AssumptionCache* AC = getAC(F);
@@ -450,18 +679,11 @@ bool StatelessToStateful::pointerIsPositiveOffsetFromKernelArgument(
         // guarantted to be DW-aligned.)
         //
         // Note that implicit arg is always aligned.
-        bool isAlignedPointee = false;
-        if (arg->isImplicitArg())
-        {
-            isAlignedPointee = true;
-        }
-        else
-        {
-            isAlignedPointee = getPointeeAlign(DL, base) >= 4 ||
-                // The intent of getKnownAlignment below is to check if any llvm.assume intrinsic provides
-                // a hint about the base pointer alignment
-                getKnownAlignment((Value*)arg->getArg(), *DL, F->getEntryBlock().getFirstNonPHI(), AC) >= 4;
-        }
+        bool isAlignedPointee =
+            arg->isImplicitArg()
+                ? true
+                : determinePointerAlignment(
+                      base, *DL, AC, F->getEntryBlock().getFirstNonPHI()) >= 4;
 
         // When compiling with patch tokens, always assume that the address
         // is aligned. This is a workaround for old OneMKL Releases. Assuming
@@ -473,15 +695,6 @@ bool StatelessToStateful::pointerIsPositiveOffsetFromKernelArgument(
         if (!m_ctx->enableZEBinary())
             isAlignedPointee = true;
 
-        // special handling
-        if (m_supportNonGEPPtr && gep == nullptr && !arg->isImplicitArg())
-        {
-            // For NonGEP ptr, do stateful only if arg isn't char*/short*
-            // (We hit bugs when allowing stateful for char*/short* arg without GEP.
-            //  Here, we simply avoid doing stateful for char*/short*.)
-            isAlignedPointee = (getPointeeAlign(DL, base) >= 4);
-        }
-
         // If m_hasBufferOffsetArg is true, the offset argument is added to
         // the final offset to make it definitely positive. Thus skip checking
         // if an offset is positive.
@@ -581,7 +794,7 @@ void StatelessToStateful::promoteIntrinsic(InstructionInfo& II)
     GenISAIntrinsic::ID const intrinID = I->getIntrinsicID();
     PointerType* pTy = IGCLLVM::getWithSamePointeeType(dyn_cast<PointerType>(II.ptr->getType()), II.getStatefulAddrSpace());
 
-    if (m_targetAddressing == TargetAddressing::BINDLESS && !WA_ForcedUsedOfBindfulMode(*m_F))
+    if (m_targetAddressing == TargetAddressing::BINDLESS)
     {
         Argument* srcOffset = m_pImplicitArgs->getNumberedImplicitArg(*m_F, ImplicitArg::BINDLESS_OFFSET, II.getBaseArgIndex());
         auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, srcOffset, pTy, "", I);
@@ -612,7 +825,7 @@ void StatelessToStateful::promoteIntrinsic(InstructionInfo& II)
         return;
     }
 
-    IGC_ASSERT(m_targetAddressing == TargetAddressing::BINDFUL || WA_ForcedUsedOfBindfulMode(*m_F));
+    IGC_ASSERT(m_targetAddressing == TargetAddressing::BINDFUL);
 
     Instruction* statefulPtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
     Instruction* statefulInst = nullptr;
@@ -703,7 +916,7 @@ void StatelessToStateful::promoteLoad(InstructionInfo& II)
 
     const DebugLoc& DL = I->getDebugLoc();
 
-    if (m_targetAddressing == TargetAddressing::BINDLESS && !WA_ForcedUsedOfBindfulMode(*m_F))
+    if (m_targetAddressing == TargetAddressing::BINDLESS)
     {
         Argument* srcOffset = m_pImplicitArgs->getNumberedImplicitArg(*m_F, ImplicitArg::BINDLESS_OFFSET, II.getBaseArgIndex());
         auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, srcOffset, pTy, "", I);
@@ -716,7 +929,7 @@ void StatelessToStateful::promoteLoad(InstructionInfo& II)
         I->eraseFromParent();
         setModuleUsesBindless();
     }
-    else if (m_targetAddressing == TargetAddressing::BINDFUL || WA_ForcedUsedOfBindfulMode(*m_F))
+    else if (m_targetAddressing == TargetAddressing::BINDFUL)
     {
         auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
         auto bindfulLoad = new LoadInst(
@@ -758,7 +971,7 @@ void StatelessToStateful::promoteStore(InstructionInfo& II)
 
     const DebugLoc& DL = I->getDebugLoc();
 
-    if (m_targetAddressing == TargetAddressing::BINDLESS && !WA_ForcedUsedOfBindfulMode(*m_F))
+    if (m_targetAddressing == TargetAddressing::BINDLESS)
     {
         Argument* srcOffset = m_pImplicitArgs->getNumberedImplicitArg(*m_F, ImplicitArg::BINDLESS_OFFSET, II.getBaseArgIndex());
         auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, srcOffset, pTy, "", I);
@@ -770,7 +983,7 @@ void StatelessToStateful::promoteStore(InstructionInfo& II)
         I->eraseFromParent();
         setModuleUsesBindless();
     }
-    else if (m_targetAddressing == TargetAddressing::BINDFUL || WA_ForcedUsedOfBindfulMode(*m_F))
+    else if (m_targetAddressing == TargetAddressing::BINDFUL)
     {
         auto newBasePtr = IntToPtrInst::Create(Instruction::IntToPtr, II.offset, pTy, "", I);
         auto bindfulStore = new StoreInst(
@@ -825,7 +1038,7 @@ void StatelessToStateful::promote()
         IGC_ASSERT(bufferPos < maxPromotionCount);
 
         unsigned statefullAddrspace = 0;
-        if (m_targetAddressing == TargetAddressing::BINDLESS && !WA_ForcedUsedOfBindfulMode(*m_F))
+        if (m_targetAddressing == TargetAddressing::BINDLESS)
         {
             statefullAddrspace =
                 IGC::EncodeAS4GFXResource(
@@ -1011,15 +1224,6 @@ void StatelessToStateful::visitStoreInst(StoreInst& I)
     }
 }
 
-bool StatelessToStateful::WA_ForcedUsedOfBindfulMode(const Function& F)
-{
-    static const std::array kernels{
-        "_ZTSZ42oneapi_kernel_integrator_intersect_closestP16KernelGlobalsGPUyyRN4sycl3_V17handlerEPKiPfiEUlNS2_7nd_itemILi1EEENS2_14kernel_handlerEE_",
-    };
-
-    return std::any_of(kernels.begin(), kernels.end(), [&F](const auto& it) { return it == F.getName(); });
-}
-
 void StatelessToStateful::findPromotableInstructions()
 {
     // fill m_promotionMap
