@@ -10,10 +10,13 @@ SPDX-License-Identifier: MIT
 
 namespace llvm {
 void initializeGenXRegionCollapsingPass(PassRegistry &);
+class GenXSubtarget;
 }
 
 struct GenXRegionCollapsingPass
     : public llvm::PassInfoMixin<GenXRegionCollapsingPass> {
+  const llvm::TargetMachine *TM;
+  explicit GenXRegionCollapsingPass(const llvm::TargetMachine *TM) : TM(TM){};
   llvm::PreservedAnalyses run(llvm::Function &F,
                               llvm::FunctionAnalysisManager &AM);
 };
