@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2024 Intel Corporation
+Copyright (C) 2020-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -9,6 +9,9 @@ SPDX-License-Identifier: MIT
 #include "SPIRVWrapper.h"
 #include "vc/Support/PassManager.h"
 
+#if LLVM_VERSION_MAJOR >= 16
+#include "GenXTargetMachine.h"
+#endif
 #include "vc/Driver/Driver.h"
 
 #include "igc/Options/Options.h"
@@ -299,6 +302,9 @@ static GenXBackendOptions createBackendOptions(const vc::CompileOptions &Opts) {
 
   BackendOpts.EnableCostModel = Opts.CollectCostInfo;
 
+  BackendOpts.DepressurizerGRFThreshold = Opts.DepressurizerGRFThreshold;
+  BackendOpts.DepressurizerFlagGRFTolerance = Opts.DepressurizerFlagGRFTolerance;
+
   return BackendOpts;
 }
 
@@ -383,7 +389,7 @@ static void optimizeIR(const vc::CompileOptions &Opts,
 #if LLVM_VERSION_MAJOR < 15
   PMBuilder.PrepareForThinLTO = false;
   PMBuilder.PrepareForLTO = false;
-#endif
+#endif // LLVM_VERSION_MAJOR < 15
   PMBuilder.RerollLoops = true;
 
   TM.adjustPassManager(PMBuilder);
@@ -400,7 +406,16 @@ static void optimizeIR(const vc::CompileOptions &Opts,
   PerFunctionPasses.doFinalization();
 
   PerModulePasses.run(M);
-#else
+#else  // LLVM_VERSION_MAJOR < 16
+
+  GenXTargetMachine *GXTM = static_cast<GenXTargetMachine *>(&TM);
+  IGC_ASSERT(GXTM);
+  if (!GXTM->getBackendConfig()) {
+    auto *BC = new GenXBackendConfig{
+        createBackendOptions(Opts),
+        createBackendData(ExtData, TM.getPointerSizeInBits(0))};
+    GXTM->setBackendConfig(BC);
+  }
 
   // Create the analysis managers.
   llvm::LoopAnalysisManager LAM;
@@ -433,7 +448,7 @@ static void optimizeIR(const vc::CompileOptions &Opts,
                     : llvm::OptimizationLevel::O0);
 
   MPM.run(M, MAM);
-#endif
+#endif // LLVM_VERSION_MAJOR < 16
 }
 
 static void populateCodeGenPassManager(const vc::CompileOptions &Opts,
@@ -798,7 +813,12 @@ static Error fillApiOptions(const opt::ArgList &ApiOptions,
   if (opt::Arg *A = ApiOptions.getLastArg(OPT_exp_register_file_size_common)) {
     StringRef V = A->getValue();
     auto MaybeGRFSize = StringSwitch<llvm::Optional<unsigned>>(V)
+                            .Case("32", 32)
+                            .Case("64", 64)
+                            .Case("96", 96)
                             .Case("128", 128)
+                            .Case("160", 160)
+                            .Case("192", 192)
                             .Case("256", 256)
                             .Default({});
     if (!MaybeGRFSize)
@@ -847,6 +867,21 @@ static Error fillApiOptions(const opt::ArgList &ApiOptions,
     Opts.StackMemSize = Result;
   }
 
+  if (opt::Arg *A = ApiOptions.getLastArg(OPT_depressurizer_grf_threshold)) {
+    StringRef Val = A->getValue();
+    unsigned Result;
+    if (Val.getAsInteger(/*Radix=*/0, Result))
+      return makeOptionError(*A, ApiOptions, /*IsInternal=*/false);
+    Opts.DepressurizerGRFThreshold = Result;
+  }
+  if (opt::Arg *A = ApiOptions.getLastArg(OPT_depressurizer_flag_grf_tolerance)) {
+    StringRef Val = A->getValue();
+    unsigned Result;
+    if (Val.getAsInteger(/*Radix=*/0, Result))
+      return makeOptionError(*A, ApiOptions, /*IsInternal=*/false);
+    Opts.DepressurizerFlagGRFTolerance = Result;
+  }
+
   return Error::success();
 }
 
