@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2024 Intel Corporation
+Copyright (C) 2017-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -21,12 +21,11 @@ SPDX-License-Identifier: MIT
 ///
 //===----------------------------------------------------------------------===//
 
-
 #include "llvmWrapper/Analysis/CallGraph.h"
 #include "llvmWrapper/IR/DerivedTypes.h"
+#include "llvmWrapper/IR/Function.h"
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/Support/Alignment.h"
-#include "llvmWrapper/IR/Function.h"
 
 #include "Probe/Assertion.h"
 
@@ -41,6 +40,7 @@ SPDX-License-Identifier: MIT
 #include "vc/Utils/GenX/TypeSize.h"
 #include "vc/Utils/General/DebugInfo.h"
 #include "vc/Utils/General/FunctionAttrs.h"
+#include "vc/Utils/General/IndexFlattener.h"
 #include "vc/Utils/General/InstRebuilder.h"
 #include "vc/Utils/General/STLExtras.h"
 #include "vc/Utils/General/Types.h"
@@ -105,7 +105,7 @@ void initializeCMABIAnalysisPass(PassRegistry &);
 void initializeCMABILegacyPass(PassRegistry &);
 void initializeCMABIPass(PassRegistry &);
 void initializeCMLowerVLoadVStorePass(PassRegistry &);
-}
+} // namespace llvm
 
 // \brief Collect necessary information for global variable localization.
 class LocalizationInfo {
@@ -422,6 +422,71 @@ template <class NodeType> Function *getFunction(NodeType i) {
   return nullptr;
 }
 
+// Analyzes the uses of a ByVal argument to determine if it is written to.
+static inline bool isReadOnlyArg(Argument &Arg) {
+  SmallVector<Use *, 8> Uses;
+  for (auto &U : Arg.uses())
+    Uses.push_back(&U);
+
+  while (!Uses.empty()) {
+    auto *U = Uses.back();
+    Uses.pop_back();
+    if (auto *SI = dyn_cast<StoreInst>(U->getUser())) {
+      if (SI->getPointerOperand() == U->get())
+        return false;
+      continue; // sutable usage
+    }
+
+    if (auto *LI = dyn_cast<LoadInst>(U->getUser()))
+      continue; // sutable usage
+
+    if (auto *GEP = dyn_cast<GetElementPtrInst>(U->getUser())) {
+      if (GEP->getPointerOperand() == U->get())
+        for (auto &GU : GEP->uses())
+          Uses.push_back(&GU);
+
+      continue;
+    }
+
+    if (auto *Cast = dyn_cast<CastInst>(U->getUser())) {
+      for (auto &CU : Cast->uses())
+        Uses.push_back(&CU);
+      continue;
+    }
+
+    return false;
+  }
+  return true;
+}
+
+// Mark SIMD-CF argument+ret functions
+static inline void CheckRMEM(Function *F) {
+  if (F->isDeclaration())
+    return;
+  for (Argument &Arg : F->args()) {
+    // Find SIMD_CF EM/RM in arguments
+    if (Arg.getType()->isVectorTy() &&
+        Arg.getType()->getScalarType()->isIntegerTy(1)) {
+      // Add a dummy attribute so that the conformance-pass can recognize a call
+      // with simd-sf
+      F->addFnAttr(vc::FunctionMD::VCSimdCFArg);
+      break;
+    }
+  }
+  auto *RetTy = F->getReturnType();
+  if (auto *ST = dyn_cast<StructType>(RetTy)) {
+    unsigned RetIdx = 0;
+    for (unsigned End = IndexFlattener::getNumElements(ST); RetIdx < End;
+         ++RetIdx) {
+      auto *Ty = IndexFlattener::getElementType(ST, RetIdx);
+      if (Ty->isVectorTy() && Ty->getScalarType()->isIntegerTy(1)) {
+        F->addFnAttr(vc::FunctionMD::VCSimdCFRet);
+        break;
+      }
+    }
+  }
+}
+
 template <class CallGraphImpl>
 bool CMABIBase<CallGraphImpl>::runOnCallGraphImpl(CallGraphImpl &SCC) {
   bool Changed = false;
@@ -452,7 +517,6 @@ bool CMABIBase<CallGraphImpl>::runOnCallGraphImpl(CallGraphImpl &SCC) {
         if (auto *CGN = ProcessNode(&*I)) {
           LocalChange = true;
           auto *NewF = getFunction(CGN);
-          ;
           SCC.getOuterRefSCC().replaceNodeFunction(*I, *NewF);
         }
       }
@@ -465,9 +529,11 @@ bool CMABIBase<CallGraphImpl>::runOnCallGraphImpl(CallGraphImpl &SCC) {
     Function *F = getFunction(&*i);
     if (!F || F->empty())
       continue;
+    CheckRMEM(F);
     for (auto &Arg : F->args()) {
       auto *ArgTy = Arg.getType();
-      if (!Arg.hasAttribute(Attribute::ByVal) || !ArgTy->isPointerTy())
+      if (!Arg.hasAttribute(Attribute::ByVal) || !ArgTy->isPointerTy() ||
+          isReadOnlyArg(Arg))
         continue;
       auto *M = F->getParent();
       auto *InsertBefore = F->getEntryBlock().getFirstNonPHI();
@@ -615,7 +681,7 @@ NodeType *CMABIBase<CallGraphImpl>::TransformKernel(Function *F) {
 
   FunctionType *NFTy = FunctionType::get(F->getReturnType(), ArgTys, false);
   IGC_ASSERT_MESSAGE((NFTy != F->getFunctionType()),
-    "type out of sync, expect bool arguments");
+                     "type out of sync, expect bool arguments");
 
   // Add any function attributes.
   AttrBuilder B(Context, PAL.getFnAttrs());
@@ -1065,12 +1131,12 @@ void CMABIBase<CallGraphImpl>::diagnoseOverlappingArgs(CallInst *CI) {
 
 char CMABILegacy::ID = 0;
 // Can't template CMABILegacy here
-INITIALIZE_PASS_BEGIN(CMABILegacy, "CMABI",
+INITIALIZE_PASS_BEGIN(CMABILegacy, "CMABILegacy",
                       "Fix ABI issues for the genx backend", false, false)
 INITIALIZE_PASS_DEPENDENCY(CallGraphWrapperPass)
 INITIALIZE_PASS_DEPENDENCY(CMABIAnalysis)
-INITIALIZE_PASS_END(CMABILegacy, "CMABI", "Fix ABI issues for the genx backend",
-                    false, false)
+INITIALIZE_PASS_END(CMABILegacy, "CMABILegacy",
+                    "Fix ABI issues for the genx backend", false, false)
 
 #if LLVM_VERSION_MAJOR >= 16
 char CMABI::ID = 0;
@@ -1169,12 +1235,12 @@ bool CMLowerVLoadVStore::runOnFunction(Function &F) {
 // instructions.
 bool CMLowerVLoadVStore::lowerLoadStore(Function &F) {
   auto M = F.getParent();
-  DenseMap<AllocaInst*, GlobalVariable*> AllocaMap;
+  DenseMap<AllocaInst *, GlobalVariable *> AllocaMap;
   // collect all the allocas that store the address of genx-volatile variable
-  for (auto& G : M->getGlobalList()) {
+  for (auto &G : M->getGlobalList()) {
     if (!G.hasAttribute("genx_volatile"))
       continue;
-    std::vector<User*> WL;
+    std::vector<User *> WL;
     for (auto UI = G.user_begin(); UI != G.user_end();) {
       auto U = *UI++;
       WL.push_back(U);
