@@ -452,6 +452,7 @@ enum class ShaderTypeMD
         bool EnableLdStCombinewithDummyLoad             = false;
         bool EnableIndependentSharedMemoryFenceFunctionality = false;
         bool NewSpillCostFunction                       = false;
+        bool EnableVRT                                  = false;
         bool ForceLargeGRFNum4RQ                        = false;
         bool DisableEUFusion                            = false;
         bool DisableFDivToFMulInvOpt                    = false;
@@ -466,6 +467,7 @@ enum class ShaderTypeMD
         bool UseBarrierControlFlowOptimization          = false;
         bool DisableDynamicRQManagement                 = false;
         unsigned Quad8InputThreshold                    = 0;
+        bool UseResourceLoopUnrollNested                = false;
     };
 
     enum class ThreadIDLayout
@@ -796,6 +798,7 @@ enum class ShaderTypeMD
         bool ModuleUsesBindless = false;
 
         llvm::MapVector<llvm::Value*, llvm::Value*> predicationMap;
+        llvm::MapVector<llvm::Value*, llvm::Value*> lifeTimeStartMap;
     };
 
     void serialize(const IGC::ModuleMetaData &moduleMD, llvm::Module* module);
