@@ -39,6 +39,8 @@ SPDX-License-Identifier: MIT
 #include "common/SIPKernels/XeHPGSIPCSRDebugBindless.h"
 #include "common/SIPKernels/XeHPCSIPCSRDebugBindless.h"
 #include "common/SIPKernels/Xe2SIPCSRDebugBindless.h"
+#include "common/SIPKernels/Xe3_G_SIPDebugBindless.h"
+#include "common/SIPKernels/wmtp/Xe3_PTL.h"
 #include "common/SIPKernels/wmtp/XE2_config_128.h"
 #include "common/SIPKernels/wmtp/XE2_config_160.h"
 
@@ -48,6 +50,225 @@ using namespace USC;
 
 namespace SIP
 {
+// wmtp PTL SIP
+struct StateSaveAreaHeaderV4 Xe3SIP_WMTP_CSRDebugBindlessDebugHeader =
+{
+    {"tssarea", 0, {4, 0, 0}, sizeof(StateSaveAreaHeaderV4) / 8, {0, 0, 0}}, // versionHeader
+    XE3_CSR_DEBUG_BINDLESS_PTL_WMTP_DATA_SIZE // total_wmtp_data_size
+};
+
+struct Xe3_G_DebugSurfaceLayout
+{
+    // The *_ALIGN fields below are padding of the SIP between
+    // the registers set.
+    static constexpr size_t GR_COUNT = 256;
+    static constexpr size_t GR_ELEMENTS = 1;
+    static constexpr size_t GR_ELEMENT_SIZE = 64;
+    static constexpr size_t GR_ALIGN = 0;
+
+    static constexpr size_t A0_COUNT = 1;
+    static constexpr size_t A0_ELEMENTS = 16;
+    static constexpr size_t A0_ELEMENT_SIZE = 2;
+    static constexpr size_t A0_ALIGN = 0x20;
+
+    static constexpr size_t F_COUNT = 4;
+    static constexpr size_t F_ELEMENTS = 2;
+    static constexpr size_t F_ELEMENT_SIZE = 2;
+    static constexpr size_t F_ALIGN = 0;
+
+    static constexpr size_t EXEC_MASK_COUNT = 1;
+    static constexpr size_t EXEC_MASK_ELEMENTS = 1;
+    static constexpr size_t EXEC_MASK_ELEMENT_SIZE = 4;
+    static constexpr size_t EXEC_MASK_ALIGN = 0;
+
+    static constexpr size_t SR_COUNT = 1;
+    static constexpr size_t SR_ELEMENTS = 5;
+    static constexpr size_t SR_ELEMENT_SIZE = 4;
+    static constexpr size_t SR_ALIGN = 0;
+
+    static constexpr size_t CR_COUNT = 1;
+    static constexpr size_t CR_ELEMENTS = 3;
+    static constexpr size_t CR_ELEMENT_SIZE = 4;
+    static constexpr size_t CR_ALIGN = 4;
+
+    static constexpr size_t N_COUNT = 1;
+    static constexpr size_t N_ELEMENTS = 3;
+    static constexpr size_t N_ELEMENT_SIZE = 4;
+    static constexpr size_t N_ALIGN = 0;
+
+    static constexpr size_t TDR_COUNT = 0; // TDR not valid for XE3
+    static constexpr size_t TDR_ELEMENTS = 0;
+    static constexpr size_t TDR_ELEMENT_SIZE = 0;
+    static constexpr size_t TDR_ALIGN = 0;
+
+    static constexpr size_t ACC_COUNT = 4;
+    static constexpr size_t ACC_ELEMENTS = 16;
+    static constexpr size_t ACC_ELEMENT_SIZE = 4; // actually defined as 33 bits, verify how these should be stored
+    static constexpr size_t ACC_ALIGN = 0xC0;
+
+    // MSG is used to access MME msg0[0-2]:ud, msg1[0-7]:ud
+    // define as 2 8 element registers
+    static constexpr size_t MSG_COUNT = 2;
+    static constexpr size_t MSG_ELEMENTS = 8;
+    static constexpr size_t MSG_ELEMENT_SIZE = 4;
+    static constexpr size_t MSG_ALIGN = 16;
+
+    static constexpr size_t MME_COUNT = 8;
+    static constexpr size_t MME_ELEMENTS = 1;
+    static constexpr size_t MME_ELEMENT_SIZE = 4;
+    static constexpr size_t MME_ALIGN = 0;
+
+    static constexpr size_t TM_COUNT = 1;
+    static constexpr size_t TM_ELEMENTS = 5;
+    static constexpr size_t TM_ELEMENT_SIZE = 4;
+    static constexpr size_t TM_ALIGN = 8;
+
+    static constexpr size_t CE_COUNT = 1;
+    static constexpr size_t CE_ELEMENTS = 1;
+    static constexpr size_t CE_ELEMENT_SIZE = 4;
+    static constexpr size_t CE_ALIGN = 0;
+
+    static constexpr size_t DBG_COUNT = 1;
+    static constexpr size_t DBG_ELEMENTS = 1;
+    static constexpr size_t DBG_ELEMENT_SIZE = 4;
+    static constexpr size_t DBG_ALIGN = 4;
+
+    static constexpr size_t VERSION_COUNT = 1;
+    static constexpr size_t VERSION_ELEMENTS = 1;
+    static constexpr size_t VERSION_ELEMENT_SIZE = 20; // sizeof(sr_ident);
+    static constexpr size_t VERSION_ALIGN = 44;        // aligning scalar/cmd
+
+    static constexpr size_t SIP_CMD_COUNT = 1;
+    static constexpr size_t SIP_CMD_ELEMENTS = 1;
+    static constexpr size_t SIP_CMD_ELEMENT_SIZE = 128;
+    static constexpr size_t SIP_CMD_ALIGN = 0;
+
+    static constexpr size_t CONTEXT_ID_COUNT = 1;
+    static constexpr size_t CONTEXT_ID_ELEMENTS = 1;
+    static constexpr size_t CONTEXT_ID_ELEMENT_SIZE = 8;
+    static constexpr size_t CONTEXT_ID_ALIGN = 8;
+
+    static constexpr size_t DBG_REG_COUNT = 1;
+    static constexpr size_t DBG_REG_ELEMENTS = 3;
+    static constexpr size_t DBG_REG_ELEMENT_SIZE = 4;
+    static constexpr size_t DBG_REG_ALIGN = 8;
+
+    static constexpr size_t FC_COUNT = 3;
+    static constexpr size_t FC_ELEMENTS = 16;
+    static constexpr size_t FC_ELEMENT_SIZE = 4;
+    static constexpr size_t FC_ALIGN = 0;
+
+    static constexpr size_t SCALAR_REG_COUNT = 1;
+    static constexpr size_t SCALAR_REG_ELEMENTS = 32;
+    static constexpr size_t SCALAR_REG_ELEMENT_SIZE = 1;
+    static constexpr size_t SCALAR_REG_ALIGN = 32;
+
+    static constexpr size_t Xe3_G_STATE_SAVE_AREA_SIZE = 0x4500;
+
+    uint8_t grf[GR_COUNT * GR_ELEMENTS * GR_ELEMENT_SIZE + GR_ALIGN];
+    uint8_t a0[A0_COUNT * A0_ELEMENTS * A0_ELEMENT_SIZE + A0_ALIGN];
+    uint8_t f[F_COUNT * F_ELEMENTS * F_ELEMENT_SIZE + F_ALIGN];
+    uint8_t sr[SR_COUNT * SR_ELEMENTS * SR_ELEMENT_SIZE + SR_ALIGN];
+    uint8_t cr[CR_COUNT * CR_ELEMENTS * CR_ELEMENT_SIZE + CR_ALIGN];
+    uint8_t n[N_COUNT * N_ELEMENTS * N_ELEMENT_SIZE + N_ALIGN];
+    uint8_t acc[ACC_COUNT * ACC_ELEMENTS * ACC_ELEMENT_SIZE + ACC_ALIGN];
+    uint8_t mme[MME_COUNT * MME_ELEMENTS * MME_ELEMENT_SIZE + MME_ALIGN];
+    uint8_t msg[MSG_COUNT * MSG_ELEMENTS * MSG_ELEMENT_SIZE + MSG_ALIGN];
+    // uint8_t tdr[TDR_COUNT * TDR_ELEMENTS * TDR_ELEMENT_SIZE + TDR_ALIGN];
+    uint8_t fc[FC_COUNT * FC_ELEMENTS * FC_ELEMENT_SIZE + FC_ALIGN];
+    uint8_t tm[TM_COUNT * TM_ELEMENTS * TM_ELEMENT_SIZE + TM_ALIGN];
+    uint8_t execmask[EXEC_MASK_COUNT * EXEC_MASK_ELEMENTS * EXEC_MASK_ELEMENT_SIZE + EXEC_MASK_ALIGN];
+    uint8_t ctx[CONTEXT_ID_COUNT * CONTEXT_ID_ELEMENTS * CONTEXT_ID_ELEMENT_SIZE + CONTEXT_ID_ALIGN];
+    uint8_t dbg_reg[DBG_REG_COUNT * DBG_REG_ELEMENTS * DBG_REG_ELEMENT_SIZE + DBG_REG_ALIGN];
+    uint8_t ce[CE_COUNT * CE_ELEMENTS * CE_ELEMENT_SIZE + CE_ALIGN];
+    uint8_t dbg[DBG_COUNT * DBG_ELEMENTS * DBG_ELEMENT_SIZE + DBG_ALIGN];
+    uint8_t version[VERSION_COUNT * VERSION_ELEMENTS * VERSION_ELEMENT_SIZE + VERSION_ALIGN];
+    uint8_t s[SCALAR_REG_COUNT * SCALAR_REG_ELEMENTS * SCALAR_REG_ELEMENT_SIZE + SCALAR_REG_ALIGN];
+    uint8_t sip_cmd[SIP_CMD_COUNT * SIP_CMD_ELEMENTS * SIP_CMD_ELEMENT_SIZE + SIP_CMD_ALIGN];
+};
+
+struct StateSaveAreaHeaderV3 Xe3_G_SIPDebugBindlessDebugHeaderV3 = {
+    {"tssarea", 0, {3, 0, 0}, sizeof(StateSaveAreaHeaderV3) / 8, {0, 0, 0}}, // versionHeader
+    {
+        // regHeader
+        0,                                                    // num_slices
+        0,                                                    // num_subslices_per_slice
+        0,                                                    // num_eus_per_subslice
+        0,                                                    // num_threads_per_eu
+        0,                                                    // state_area_offset
+        Xe3_G_DebugSurfaceLayout::Xe3_G_STATE_SAVE_AREA_SIZE, // state_save_size
+        0,                                                    // slm_area_offset
+        0,                                                    // slm_bank_size
+        0,                                                    // slm_bank_valid
+        offsetof(struct Xe3_G_DebugSurfaceLayout, version),   // sr_magic_offset
+        0,                                                    // fifo_offset
+        0,                                                    // fifo_size,
+        0,                                                    // fifo_head
+        0,                                                    // fifo_tail
+        0,                                                    // fifo_version
+        {0},                                                  // reserved1
+        0,                                                    // sip_flags
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, grf), Xe3_G_DebugSurfaceLayout::GR_COUNT,
+         Xe3_G_DebugSurfaceLayout::GR_ELEMENTS *Xe3_G_DebugSurfaceLayout::GR_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::GR_ELEMENTS *Xe3_G_DebugSurfaceLayout::GR_ELEMENT_SIZE}, // grf
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, a0), Xe3_G_DebugSurfaceLayout::A0_COUNT,
+         Xe3_G_DebugSurfaceLayout::A0_ELEMENTS *Xe3_G_DebugSurfaceLayout::A0_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::A0_ELEMENTS *Xe3_G_DebugSurfaceLayout::A0_ELEMENT_SIZE}, // addr
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, f), Xe3_G_DebugSurfaceLayout::F_COUNT,
+         Xe3_G_DebugSurfaceLayout::F_ELEMENTS *Xe3_G_DebugSurfaceLayout::F_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::F_ELEMENTS *Xe3_G_DebugSurfaceLayout::F_ELEMENT_SIZE}, // flag
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, execmask), Xe3_G_DebugSurfaceLayout::EXEC_MASK_COUNT,
+         Xe3_G_DebugSurfaceLayout::EXEC_MASK_ELEMENTS *Xe3_G_DebugSurfaceLayout::EXEC_MASK_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::EXEC_MASK_ELEMENTS *Xe3_G_DebugSurfaceLayout::EXEC_MASK_ELEMENT_SIZE}, // emask
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, sr), Xe3_G_DebugSurfaceLayout::SR_COUNT,
+         Xe3_G_DebugSurfaceLayout::SR_ELEMENTS *Xe3_G_DebugSurfaceLayout::SR_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::SR_ELEMENTS *Xe3_G_DebugSurfaceLayout::SR_ELEMENT_SIZE}, // sr
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, cr), Xe3_G_DebugSurfaceLayout::CR_COUNT,
+         Xe3_G_DebugSurfaceLayout::CR_ELEMENTS *Xe3_G_DebugSurfaceLayout::CR_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::CR_ELEMENTS *Xe3_G_DebugSurfaceLayout::CR_ELEMENT_SIZE}, // cr
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, n), Xe3_G_DebugSurfaceLayout::N_COUNT,
+         Xe3_G_DebugSurfaceLayout::N_ELEMENTS *Xe3_G_DebugSurfaceLayout::N_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::N_ELEMENTS *Xe3_G_DebugSurfaceLayout::N_ELEMENT_SIZE}, // notification
+        {0, 0, // offsetof(struct Xe3_G_DebugSurfaceLayout, tdr), Xe3_G_DebugSurfaceLayout::TDR_COUNT,
+         Xe3_G_DebugSurfaceLayout::TDR_ELEMENTS *Xe3_G_DebugSurfaceLayout::TDR_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::TDR_ELEMENTS *Xe3_G_DebugSurfaceLayout::TDR_ELEMENT_SIZE}, // tdr
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, acc), Xe3_G_DebugSurfaceLayout::ACC_COUNT,
+         Xe3_G_DebugSurfaceLayout::ACC_ELEMENTS *Xe3_G_DebugSurfaceLayout::ACC_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::ACC_ELEMENTS *Xe3_G_DebugSurfaceLayout::ACC_ELEMENT_SIZE}, // acc
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, mme), Xe3_G_DebugSurfaceLayout::MME_COUNT,
+         Xe3_G_DebugSurfaceLayout::MME_ELEMENTS *Xe3_G_DebugSurfaceLayout::MME_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::MME_ELEMENTS *Xe3_G_DebugSurfaceLayout::MME_ELEMENT_SIZE}, // mme
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, ce), Xe3_G_DebugSurfaceLayout::CE_COUNT,
+         Xe3_G_DebugSurfaceLayout::CE_ELEMENTS *Xe3_G_DebugSurfaceLayout::CE_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::CE_ELEMENTS *Xe3_G_DebugSurfaceLayout::CE_ELEMENT_SIZE}, // ce
+        {0, 0, 0, 0},                                                                       // sp
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, sip_cmd), Xe3_G_DebugSurfaceLayout::SIP_CMD_COUNT,
+         Xe3_G_DebugSurfaceLayout::SIP_CMD_ELEMENTS *Xe3_G_DebugSurfaceLayout::SIP_CMD_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::SIP_CMD_ELEMENTS *Xe3_G_DebugSurfaceLayout::SIP_CMD_ELEMENT_SIZE}, // cmd
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, tm), Xe3_G_DebugSurfaceLayout::TM_COUNT,
+         Xe3_G_DebugSurfaceLayout::TM_ELEMENTS *Xe3_G_DebugSurfaceLayout::TM_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::TM_ELEMENTS *Xe3_G_DebugSurfaceLayout::TM_ELEMENT_SIZE}, // tm
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, fc), Xe3_G_DebugSurfaceLayout::FC_COUNT,
+         Xe3_G_DebugSurfaceLayout::FC_ELEMENTS *Xe3_G_DebugSurfaceLayout::FC_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::FC_ELEMENTS *Xe3_G_DebugSurfaceLayout::FC_ELEMENT_SIZE}, // FC
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, dbg), Xe3_G_DebugSurfaceLayout::DBG_COUNT,
+         Xe3_G_DebugSurfaceLayout::DBG_ELEMENTS *Xe3_G_DebugSurfaceLayout::DBG_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::DBG_ELEMENTS *Xe3_G_DebugSurfaceLayout::DBG_ELEMENT_SIZE}, // dbg
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, ctx), Xe3_G_DebugSurfaceLayout::CONTEXT_ID_COUNT,
+         Xe3_G_DebugSurfaceLayout::CONTEXT_ID_ELEMENTS *Xe3_G_DebugSurfaceLayout::CONTEXT_ID_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::CONTEXT_ID_ELEMENTS
+             *Xe3_G_DebugSurfaceLayout::CONTEXT_ID_ELEMENT_SIZE}, // context id
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, dbg_reg), Xe3_G_DebugSurfaceLayout::DBG_REG_COUNT,
+         Xe3_G_DebugSurfaceLayout::DBG_REG_ELEMENTS *Xe3_G_DebugSurfaceLayout::DBG_REG_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::DBG_REG_ELEMENTS *Xe3_G_DebugSurfaceLayout::DBG_REG_ELEMENT_SIZE}, // dbg registers
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, s), Xe3_G_DebugSurfaceLayout::SCALAR_REG_COUNT, // scalar registers
+         Xe3_G_DebugSurfaceLayout::SCALAR_REG_ELEMENTS *Xe3_G_DebugSurfaceLayout::SCALAR_REG_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::SCALAR_REG_ELEMENTS *Xe3_G_DebugSurfaceLayout::SCALAR_REG_ELEMENT_SIZE},
+        {offsetof(struct Xe3_G_DebugSurfaceLayout, msg), Xe3_G_DebugSurfaceLayout::MSG_COUNT,
+         Xe3_G_DebugSurfaceLayout::MSG_ELEMENTS *Xe3_G_DebugSurfaceLayout::MSG_ELEMENT_SIZE * 8,
+         Xe3_G_DebugSurfaceLayout::MSG_ELEMENTS *Xe3_G_DebugSurfaceLayout::MSG_ELEMENT_SIZE} // msg
+    }
+};
 
 
 // Debug surface area for all XE2 architectures
@@ -810,6 +1031,7 @@ bool SIPSuppoertedOnPlatformFamily(const GFXCORE_FAMILY& family)
     case IGFX_XE_HPG_CORE:
     case IGFX_XE_HPC_CORE:
     case IGFX_XE2_HPG_CORE:
+    case IGFX_XE3_CORE:
         return true;
     default:
         return false;
@@ -1110,6 +1332,30 @@ void populateSIPKernelInfo(const IGC::CPlatform &platform,
 
      }
 
+    // PTL wmtp Sip
+    SIPKernelInfo[XE3_CSR_DEBUG_BINDLESS] = std::make_tuple(
+            (void *)&Xe3_PTL, (int)sizeof(Xe3_PTL),
+            (void *)&Xe3SIP_WMTP_CSRDebugBindlessDebugHeader,  (int)sizeof(Xe3SIP_WMTP_CSRDebugBindlessDebugHeader));
+
+    // Xe3 / Xe3G
+    SIPKernelInfo[XE3G_DEBUG_BINDLESS] = std::make_tuple((void*)&Xe3_G_SIPDebugBindless,
+              (int)sizeof(Xe3_G_SIPDebugBindless), (void*)&Xe3_G_SIPDebugBindlessDebugHeaderV3,
+              (int)sizeof(Xe3_G_SIPDebugBindlessDebugHeaderV3));
+
+    Xe3_G_SIPDebugBindlessDebugHeaderV3.regHeader.num_threads_per_eu = 0;
+
+    if (sysInfo.EUCount != 0)
+        Xe3_G_SIPDebugBindlessDebugHeaderV3.regHeader.num_threads_per_eu = (sysInfo.ThreadCount / sysInfo.EUCount);
+
+    if (sizeof(StateSaveAreaHeader) % 16)
+        Xe3_G_SIPDebugBindlessDebugHeaderV3.regHeader.state_area_offset =
+            16 - sizeof(StateSaveAreaHeader) % 16;
+
+    Xe3_G_SIPDebugBindlessDebugHeaderV3.regHeader.num_slices = sysInfo.MaxSlicesSupported;
+    Xe3_G_SIPDebugBindlessDebugHeaderV3.regHeader.num_subslices_per_slice =
+            (sysInfo.MaxSlicesSupported > 0 ? (sysInfo.MaxSubSlicesSupported / sysInfo.MaxSlicesSupported) : sysInfo.MaxSubSlicesSupported);
+    Xe3_G_SIPDebugBindlessDebugHeaderV3.regHeader.num_eus_per_subslice = sysInfo.MaxEuPerSubSlice;
+
 }
 
 CGenSystemInstructionKernelProgram* CGenSystemInstructionKernelProgram::Create(
@@ -1292,6 +1538,18 @@ CGenSystemInstructionKernelProgram* CGenSystemInstructionKernelProgram::Create(
         }
         break;
     }
+
+    case IGFX_XE3_CORE:
+        if (mode & SYSTEM_THREAD_MODE_DEBUG)
+        {
+            SIPIndex = XE3G_DEBUG_BINDLESS;
+        }
+        else if(mode & SYSTEM_THREAD_MODE_CSR)
+        {
+            SIPIndex = XE3_CSR_DEBUG_BINDLESS;
+        }
+        break;
+
     default:
         IGC_ASSERT(0);
         break;
