@@ -108,6 +108,7 @@ public:
 enum class forbiddenKind {
   FBD_ADDR = 0,
   FBD_FLAG = 1,
+  FBD_SCALAR,
   FBD_RESERVEDGRF,
   FBD_EOT,
   FBD_LASTGRF,
@@ -1307,6 +1308,8 @@ public:
         getForbiddenVectorSize(G4_ADDRESS));
     forbiddenVec[(size_t)forbiddenKind::FBD_FLAG].resize(
         getForbiddenVectorSize(G4_FLAG));
+    forbiddenVec[(size_t)forbiddenKind::FBD_SCALAR].resize(
+        getForbiddenVectorSize(G4_SCALAR));
   };
   ~ForbiddenRegs(){};
 
@@ -1402,6 +1405,7 @@ private:
   static const RAVarInfo defaultValues;
   std::vector<RAVarInfo> vars;
   std::vector<G4_Declare *> UndeclaredVars;
+  std::vector<G4_Declare *> UndefinedCmpVars;
 
   // fake declares for each GRF reg, used by HRA
   // note only GRFs that are used by LRA get a declare
@@ -1535,7 +1539,7 @@ public:
   InterferenceMatrixStorage intfStorage;
   IncrementalRA incRA;
 
-
+  bool didGRFIncrease = false;
   bool avoidBundleConflict = false;
 
   unsigned getSubRetLoc(const G4_BB *bb) {
@@ -1614,6 +1618,7 @@ public:
   void addSpillCodeInBB(G4_BB *bb) { BBsWithSpillCode.insert(bb); }
 
   void addUndefinedDcl(G4_Declare *dcl) { UndeclaredVars.push_back(dcl); }
+  void addUndefinedCmpDcl(G4_Declare *dcl) { UndefinedCmpVars.push_back(dcl); }
 
   bool isUndefinedDcl(const G4_Declare *dcl) const {
     return std::find(UndeclaredVars.begin(), UndeclaredVars.end(), dcl) !=
@@ -1686,6 +1691,7 @@ public:
       setBBId(dcl, UINT_MAX);
       resetLocalLR(dcl);
     }
+    UndefinedCmpVars.clear();
   }
 
   void clearLocalLiveRanges() {
@@ -1979,6 +1985,7 @@ public:
   void reportSpillInfo(const LivenessAnalysis &liveness,
                        const GraphColor &coloring) const;
   static uint32_t getRefCount(int loopNestLevel);
+  bool canIncreaseGRF(unsigned spillSize, bool infCostSpilled);
   void updateSubRegAlignment(G4_SubReg_Align subAlign);
   bool isChannelSliced();
   // Used by LRA/GRA/hybrid RA
@@ -2005,6 +2012,8 @@ public:
                       std::unordered_set<G4_INST *> &group);
   void addrRegAlloc();
   void flagRegAlloc();
+  void scalarRegAlloc();
+  void selectScalarCandidates();
   void fastRADecision();
   bool tryHybridRA();
   bool hybridRA(LocalRA &lra);
@@ -2044,6 +2053,7 @@ public:
   void addCalleeSavePseudoCode();
   void addStoreRestoreToReturn();
   void storeCEInProlog();
+  void setUndefinedVarCmp();
   void markGraphBlockLocalVars();
   void verifyRA(LivenessAnalysis &liveAnalysis);
   void verifySpillFill();
@@ -2166,6 +2176,7 @@ private:
   void writeVerboseStatsNumVars(LivenessAnalysis &liveAnalysis,
                                 FINALIZER_INFO *jitInfo);
   void writeVerboseRPEStats(RPE &rpe);
+  bool VRTIncreasedGRF(GraphColor &coloring);
   void splitOnSpill(bool fastCompile, GraphColor &coloring,
                     LivenessAnalysis &livenessAnalysis);
   bool convertToFailSafe(bool reserveSpillReg, GraphColor &coloring,
