@@ -311,6 +311,9 @@ static inline GED_MODEL lowerPlatform(Platform platform) {
   case Platform::XE2:
     pltf = GED_MODEL_XE2;
     break;
+  case Platform::XE3:
+    pltf = GED_MODEL_XE3;
+    break;
   default:
     break;
   }
@@ -1051,6 +1054,9 @@ static inline GED_ARCH_REG lowerArchReg(RegName type) {
   case RegName::ARF_SP:
     archReg = GED_ARCH_REG_sp;
     break;
+  case RegName::ARF_S:
+    archReg = GED_ARCH_REG_s;
+    break;
   case RegName::ARF_SR:
     archReg = GED_ARCH_REG_sr0;
     break;
