@@ -420,6 +420,7 @@ typedef enum {
   ISA_RESERVED_9A = 0x9A,
   ISA_INVM = 0x9B,
   ISA_RSQTM = 0x9C,
+  ISA_RESERVED_9D = 0x9D,
   ISA_NUM_OPCODE,
   ISA_OPCODE_ENUM_SIZE = 0xFF
 } ISA_Opcode;
@@ -527,6 +528,7 @@ typedef enum {
   Xe_PVC,
   Xe_PVCXT,
   Xe2,
+  Xe3,
   ALL
 } TARGET_PLATFORM;
 
@@ -922,6 +924,8 @@ enum LSC_OP {
   LSC_APNDCTR_ATOMIC_SUB = 0x29,
   LSC_APNDCTR_ATOMIC_STORE = 0x2A,
 
+  LSC_LOAD_QUAD_MSRT = 0x31,
+  LSC_STORE_QUAD_MSRT = 0x32,
   LSC_INVALID = 0xFFFFFFFF,
 };
 
